#!/usr/bin/env python
#
# Copyright 2007,2010,2012 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

from gnuradio import gr, gr_unittest, analog, blocks
import numpy


class test_fastnoise_source(gr_unittest.TestCase):

    def setUp(self):

        self.num = 2**22
        self.num_items = 10**6
        self.default_args = {"samples": self.num, "seed": 43, "ampl": 1}

    def tearDown(self):
        pass

    def run_test_real(self, form):
        """ Run test case with float input/output
        """
        tb = gr.top_block()
        src = analog.fastnoise_source_f(type=form, **self.default_args)
        head = blocks.head(
            nitems=self.num_items,
            sizeof_stream_item=gr.sizeof_float)
        sink = blocks.vector_sink_f()
        tb.connect(src, head, sink)
        tb.run()
        return numpy.array(sink.data())

    def run_test_complex(self, form):
        """ Run test case with complex input/output
        """
        tb = gr.top_block()
        src = analog.fastnoise_source_c(type=form, **self.default_args)
        head = blocks.head(
            nitems=self.num_items,
            sizeof_stream_item=gr.sizeof_gr_complex)
        sink = blocks.vector_sink_c()
        tb.connect(src, head, sink)
        tb.run()
        return numpy.array(sink.data())

    def test_001_real_uniform_moments(self):

        data = self.run_test_real(analog.GR_UNIFORM)

        self.assertAlmostEqual(min(data), -1, places=4)
        self.assertAlmostEqual(max(data), 1, places=4)

        # mean, variance
        self.assertAlmostEqual(data.mean(), 0, places=2)
        self.assertAlmostEqual(data.var(), (1 - (-1))**2. / 12, places=3)

    def test_001_real_gaussian_moments(self):
        data = self.run_test_real(analog.GR_GAUSSIAN)

        # mean, variance
        self.assertAlmostEqual(data.mean(), 0, places=2)
        self.assertAlmostEqual(data.var(), 1, places=2)

    def test_001_real_laplacian_moments(self):
        data = self.run_test_real(analog.GR_LAPLACIAN)

        # mean, variance
        self.assertAlmostEqual(data.mean(), 0, places=2)
        self.assertAlmostEqual(data.var(), 2, places=2)

    def test_001_complex_uniform_moments(self):
        data = self.run_test_complex(analog.GR_UNIFORM)

        # mean, variance
        self.assertAlmostEqual(data.real.mean(), 0, places=2)
        self.assertAlmostEqual(data.real.var(), 0.5 *
                               (1 - (-1))**2. / 12, places=3)

        self.assertAlmostEqual(data.imag.mean(), 0, places=2)
        self.assertAlmostEqual(data.imag.var(), 0.5 *
                               (1 - (-1))**2. / 12, places=3)

    def test_001_complex_gaussian_moments(self):
        data = self.run_test_complex(analog.GR_GAUSSIAN)

        # mean, variance
        self.assertAlmostEqual(data.real.mean(), 0, places=2)
        self.assertAlmostEqual(data.real.var(), 0.5, places=2)

        self.assertAlmostEqual(data.imag.mean(), 0, places=2)
        self.assertAlmostEqual(data.imag.var(), 0.5, places=2)

    def test_002_real_uniform_reproducibility(self):
        data1 = self.run_test_real(analog.GR_UNIFORM)
        data2 = self.run_test_real(analog.GR_UNIFORM)

        # It's pseudoramdo thus must be equal
        self.assertTrue(numpy.array_equal(data1, data2))

    def test_002_real_gaussian_reproducibility(self):
        data1 = self.run_test_real(analog.GR_GAUSSIAN)
        data2 = self.run_test_real(analog.GR_GAUSSIAN)

        self.assertTrue(numpy.array_equal(data1, data2))

    def test_003_real_uniform_pool(self):
        src = analog.fastnoise_source_f(
            type=analog.GR_UNIFORM, **self.default_args)
        src2 = analog.fastnoise_source_f(
            type=analog.GR_UNIFORM, **self.default_args)
        self.assertTrue(
            numpy.array_equal(
                numpy.array(
                    src.samples()), numpy.array(
                    src2.samples())))

    def test_003_real_gaussian_pool(self):
        src = analog.fastnoise_source_f(
            type=analog.GR_GAUSSIAN, **self.default_args)
        src2 = analog.fastnoise_source_f(
            type=analog.GR_GAUSSIAN, **self.default_args)
        self.assertTrue(
            numpy.array_equal(
                numpy.array(
                    src.samples()), numpy.array(
                    src2.samples())))

    def test_003_cmplx_gaussian_pool(self):
        src = analog.fastnoise_source_c(
            type=analog.GR_GAUSSIAN, **self.default_args)
        src2 = analog.fastnoise_source_c(
            type=analog.GR_GAUSSIAN, **self.default_args)
        self.assertTrue(
            numpy.array_equal(
                numpy.array(
                    src.samples()), numpy.array(
                    src2.samples())))

    def test_003_cmplx_uniform_pool(self):
        src = analog.fastnoise_source_c(
            type=analog.GR_UNIFORM, **self.default_args)
        src2 = analog.fastnoise_source_c(
            type=analog.GR_UNIFORM, **self.default_args)
        self.assertTrue(
            numpy.array_equal(
                numpy.array(
                    src.samples()), numpy.array(
                    src2.samples())))

    def test_003_real_laplacian_pool(self):
        src = analog.fastnoise_source_f(
            type=analog.GR_LAPLACIAN, **self.default_args)
        src2 = analog.fastnoise_source_f(
            type=analog.GR_LAPLACIAN, **self.default_args)
        self.assertTrue(
            numpy.array_equal(
                numpy.array(
                    src.samples()), numpy.array(
                    src2.samples())))


if __name__ == '__main__':
    gr_unittest.run(test_fastnoise_source)



#!/usr/bin/env python
#
# Copyright 2020 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# GNU Radio is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# GNU Radio is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with GNU Radio; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
#
#


from gnuradio import gr, gr_unittest, blocks
import pmt
import os


class qa_stream_demux(gr_unittest.TestCase):

    def setUp(self):
        os.environ['GR_CONF_CONTROLPORT_ON'] = 'False'
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def help_stream_2ff(self, N, stream_sizes):
        v = blocks.vector_source_f(N * [1, ] + N * [2, ], False)

        demux = blocks.stream_demux(gr.sizeof_float, stream_sizes)

        dst0 = blocks.vector_sink_f()
        dst1 = blocks.vector_sink_f()

        self.tb.connect(v, demux)
        self.tb.connect((demux, 0), dst0)
        self.tb.connect((demux, 1), dst1)
        self.tb.run()

        return (dst0.data(), dst1.data())

    def help_stream_ramp_2ff(self, N, stream_sizes):
        r = list(range(N)) + list(reversed(range(N)))

        v = blocks.vector_source_f(r, False)

        demux = blocks.stream_demux(gr.sizeof_float, stream_sizes)

        dst0 = blocks.vector_sink_f()
        dst1 = blocks.vector_sink_f()

        self.tb.connect(v, demux)
        self.tb.connect((demux, 0), dst0)
        self.tb.connect((demux, 1), dst1)
        self.tb.run()

        return (dst0.data(), dst1.data())

    def help_stream_tag_propagation(self, N, stream_sizes):
        src_data = (stream_sizes[0] * [1, ] + stream_sizes[1] *
                    [2, ] + stream_sizes[2] * [3, ]) * N

        src = blocks.vector_source_f(src_data, False)

        tag_stream1 = blocks.stream_to_tagged_stream(gr.sizeof_float, 1,
                                                     stream_sizes[0], 'src1')
        tag_stream2 = blocks.stream_to_tagged_stream(gr.sizeof_float, 1,
                                                     stream_sizes[1], 'src2')
        tag_stream3 = blocks.stream_to_tagged_stream(gr.sizeof_float, 1,
                                                     stream_sizes[2], 'src3')

        demux = blocks.stream_demux(gr.sizeof_float, stream_sizes)
        dst0 = blocks.vector_sink_f()
        dst1 = blocks.vector_sink_f()
        dst2 = blocks.vector_sink_f()

        self.tb.connect(src, tag_stream1)
        self.tb.connect(tag_stream1, tag_stream2)
        self.tb.connect(tag_stream2, tag_stream3)
        self.tb.connect(tag_stream3, demux)
        self.tb.connect((demux, 0), dst0)
        self.tb.connect((demux, 1), dst1)
        self.tb.connect((demux, 2), dst2)
        self.tb.run()

        return (dst0, dst1, dst2)

    def test_stream_2NN_ff(self):
        N = 40
        stream_sizes = [10, 10]
        result_data = self.help_stream_2ff(N, stream_sizes)

        exp_data0 = [1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0,
                     1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0,
                     2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0,
                     2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0]
        exp_data1 = [1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0,
                     1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0,
                     2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0,
                     2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0]

        self.assertEqual(exp_data0, result_data[0])
        self.assertEqual(exp_data1, result_data[1])

    def test_stream_ramp_2NN_ff(self):
        N = 40
        stream_sizes = [10, 10]
        result_data = self.help_stream_ramp_2ff(N, stream_sizes)

        exp_data0 = [
            0.0,
            1.0,
            2.0,
            3.0,
            4.0,
            5.0,
            6.0,
            7.0,
            8.0,
            9.0,
            20.0,
            21.0,
            22.0,
            23.0,
            24.0,
            25.0,
            26.0,
            27.0,
            28.0,
            29.0,
            39.0,
            38.0,
            37.0,
            36.0,
            35.0,
            34.0,
            33.0,
            32.0,
            31.0,
            30.0,
            19.0,
            18.0,
            17.0,
            16.0,
            15.0,
            14.0,
            13.0,
            12.0,
            11.0,
            10.0]
        exp_data1 = [
            10.0,
            11.0,
            12.0,
            13.0,
            14.0,
            15.0,
            16.0,
            17.0,
            18.0,
            19.0,
            30.0,
            31.0,
            32.0,
            33.0,
            34.0,
            35.0,
            36.0,
            37.0,
            38.0,
            39.0,
            29.0,
            28.0,
            27.0,
            26.0,
            25.0,
            24.0,
            23.0,
            22.0,
            21.0,
            20.0,
            9.0,
            8.0,
            7.0,
            6.0,
            5.0,
            4.0,
            3.0,
            2.0,
            1.0,
            0.0]

        self.assertEqual(exp_data0, result_data[0])
        self.assertEqual(exp_data1, result_data[1])

    def test_stream_2NM_ff(self):
        N = 40
        stream_sizes = [7, 9]
        self.help_stream_2ff(N, stream_sizes)

        result_data = self.help_stream_2ff(N, stream_sizes)

        exp_data0 = [1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0,
                     1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0,
                     1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0,
                     2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0,
                     2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0]

        exp_data1 = [1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0,
                     1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0,
                     1.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0,
                     2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0,
                     2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0]

        self.assertEqual(exp_data0, result_data[0])
        self.assertEqual(exp_data1, result_data[1])

    def test_stream_2MN_ff(self):
        N = 37
        stream_sizes = [7, 9]
        self.help_stream_2ff(N, stream_sizes)

        result_data = self.help_stream_2ff(N, stream_sizes)

        exp_data0 = [1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0,
                     1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0,
                     1.0, 1.0, 1.0, 1.0, 1.0, 2.0, 2.0,
                     2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0,
                     2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0]

        exp_data1 = [1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0,
                     1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0,
                     2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0,
                     2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0,
                     2.0, 2.0, 2.0]

        self.assertEqual(exp_data0, result_data[0])
        self.assertEqual(exp_data1, result_data[1])

    def test_stream_2N0_ff(self):
        N = 30
        stream_sizes = [7, 0]
        self.help_stream_2ff(N, stream_sizes)

        result_data = self.help_stream_2ff(N, stream_sizes)

        exp_data0 = [1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0,
                     1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0,
                     1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0,
                     1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0,
                     1.0, 1.0, 2.0, 2.0, 2.0, 2.0, 2.0,
                     2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0,
                     2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0,
                     2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0,
                     2.0, 2.0, 2.0, 2.0]
        exp_data1 = []

        self.assertEqual(exp_data0, result_data[0])
        self.assertEqual(exp_data1, result_data[1])

    def test_stream_20N_ff(self):
        N = 30
        stream_sizes = [0, 9]
        self.help_stream_2ff(N, stream_sizes)

        result_data = self.help_stream_2ff(N, stream_sizes)

        exp_data0 = []
        exp_data1 = [1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0,
                     1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0,
                     1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0,
                     1.0, 1.0, 1.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0,
                     2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0,
                     2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0, 2.0,
                     2.0, 2.0, 2.0, 2.0, 2.0, 2.0]

        self.assertEqual(exp_data0, result_data[0])
        self.assertEqual(exp_data1, result_data[1])

    def test_largeN_ff(self):
        stream_sizes = [3, 8191]
        r0 = [1.0, ] * stream_sizes[0]
        r1 = [2.0, ] * stream_sizes[1]
        v = blocks.vector_source_f(r0 + r1, repeat=False)
        demux = blocks.stream_demux(gr.sizeof_float, stream_sizes)
        dst0 = blocks.vector_sink_f()
        dst1 = blocks.vector_sink_f()
        self.tb.connect(v, demux)
        self.tb.connect((demux, 0), dst0)
        self.tb.connect((demux, 1), dst1)
        self.tb.run()
        self.assertEqual(r0, dst0.data())
        self.assertEqual(r1, dst1.data())

    def test_tag_propagation(self):
        N = 10  # Block length
        stream_sizes = [1, 2, 3]

        expected_result0 = N * (stream_sizes[0] * [1, ])
        expected_result1 = N * (stream_sizes[1] * [2, ])
        expected_result2 = N * (stream_sizes[2] * [3, ])

        # check the data
        (result0, result1, result2) = self.help_stream_tag_propagation(N, stream_sizes)
        self.assertFloatTuplesAlmostEqual(
            expected_result0, result0.data(), places=6)
        self.assertFloatTuplesAlmostEqual(
            expected_result1, result1.data(), places=6)
        self.assertFloatTuplesAlmostEqual(
            expected_result2, result2.data(), places=6)

        # check the tags - result0
        tags = result0.tags()
        expected_tag_offsets_src1 = list(
            range(0, stream_sizes[0] * N, stream_sizes[0]))
        expected_tag_offsets_src2 = list(
            range(0, stream_sizes[0] * N, stream_sizes[0]))
        expected_tag_offsets_src3 = list(
            range(0, stream_sizes[0] * N, stream_sizes[0]))
        tags_src1 = [
            tag for tag in tags if pmt.eq(
                tag.key, pmt.intern('src1'))]
        tags_src2 = [
            tag for tag in tags if pmt.eq(
                tag.key, pmt.intern('src2'))]
        tags_src3 = [
            tag for tag in tags if pmt.eq(
                tag.key, pmt.intern('src3'))]
        for i in range(len(expected_tag_offsets_src1)):
            self.assertEqual(expected_tag_offsets_src1[i], tags_src1[i].offset)
        for i in range(len(expected_tag_offsets_src2)):
            self.assertEqual(expected_tag_offsets_src2[i], tags_src2[i].offset)
        for i in range(len(expected_tag_offsets_src3)):
            self.assertEqual(expected_tag_offsets_src3[i], tags_src3[i].offset)

        # check the tags - result1
        tags = result1.tags()
        expected_tag_offsets_src1 = list(
            range(0, stream_sizes[1] * N, stream_sizes[0]))
        expected_tag_offsets_src2 = list(
            range(1, stream_sizes[1] * N, stream_sizes[1]))
        expected_tag_offsets_src3 = list()
        tags_src1 = [
            tag for tag in tags if pmt.eq(
                tag.key, pmt.intern('src1'))]
        tags_src2 = [
            tag for tag in tags if pmt.eq(
                tag.key, pmt.intern('src2'))]
        tags_src3 = [
            tag for tag in tags if pmt.eq(
                tag.key, pmt.intern('src3'))]
        for i in range(len(expected_tag_offsets_src1)):
            self.assertEqual(expected_tag_offsets_src1[i], tags_src1[i].offset)
        for i in range(len(expected_tag_offsets_src2)):
            self.assertEqual(expected_tag_offsets_src2[i], tags_src2[i].offset)
        for i in range(len(expected_tag_offsets_src3)):
            self.assertEqual(expected_tag_offsets_src3[i], tags_src3[i].offset)

        # check the tags - result2
        tags = result2.tags()
        expected_tag_offsets_src1 = list(
            range(0, stream_sizes[2] * N, stream_sizes[0]))
        expected_tag_offsets_src2 = list(
            range(1, stream_sizes[2] * N, stream_sizes[2]))
        expected_tag_offsets_src3 = list(
            range(0, stream_sizes[2] * N, stream_sizes[2]))
        tags_src1 = [
            tag for tag in tags if pmt.eq(
                tag.key, pmt.intern('src1'))]
        tags_src2 = [
            tag for tag in tags if pmt.eq(
                tag.key, pmt.intern('src2'))]
        tags_src3 = [
            tag for tag in tags if pmt.eq(
                tag.key, pmt.intern('src3'))]
        for i in range(len(expected_tag_offsets_src1)):
            self.assertEqual(expected_tag_offsets_src1[i], tags_src1[i].offset)
        for i in range(len(expected_tag_offsets_src2)):
            self.assertEqual(expected_tag_offsets_src2[i], tags_src2[i].offset)
        for i in range(len(expected_tag_offsets_src3)):
            self.assertEqual(expected_tag_offsets_src3[i], tags_src3[i].offset)


if __name__ == '__main__':
    gr_unittest.run(qa_stream_demux)



#!/usr/bin/env python
#
# Copyright 2016 Free Software Foundation, Inc.
# Copyright 2021 Marcus Müller
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks


class test_logger (gr_unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def set_and_assert_log_level(self, block, level, ref=None):
        if ref is None:
            ref = level
        block.set_log_level(level)
        self.assertEqual(block.log_level(), ref)

    def test_log_level_for_block(self):
        # Test the python API for getting and setting log levels of individual block
        # go through all of the documented log_levels
        ns = blocks.null_source(1)
        self.set_and_assert_log_level(ns, "debug")
        self.set_and_assert_log_level(ns, "info")
        self.set_and_assert_log_level(ns, "notice", "info")
        self.set_and_assert_log_level(ns, "warn", "warning")
        self.set_and_assert_log_level(ns, "warning")
        self.set_and_assert_log_level(ns, "error")
        self.set_and_assert_log_level(ns, "crit", "critical")
        self.set_and_assert_log_level(ns, "critical")
        self.set_and_assert_log_level(ns, "alert", "critical")
        self.set_and_assert_log_level(ns, "emerg", "critical")
        # There's a couple of special cases. "off" == "notset" (specific to gr)
        # and "fatal" == "emerg" (built in to log4cpp)
        ns.set_log_level("off")
        self.assertEqual(ns.log_level(), "off")

    def test_log_level_for_tb(self):
        # Test the python API for getting and setting log levels for a
        # top_block
        nsrc = blocks.null_source(4)
        nsnk = blocks.null_sink(4)
        # Set all log levels to a known state
        nsrc.set_log_level("debug")
        nsnk.set_log_level("debug")
        tb = gr.top_block()
        tb.connect(nsrc, nsnk)
        # confirm that the tb has log_level of first block
        self.assertEqual(tb.log_level(), "debug")
        # confirm that changing tb log_level propagates to connected blocks
        tb.set_log_level("critical")
        self.assertEqual(tb.log_level(), "critical")
        self.assertEqual(nsrc.log_level(), "critical")
        self.assertEqual(nsnk.log_level(), "critical")

    def test_log_level_for_hier_block(self):
        # Test the python API for getting and setting log levels for hier
        # blocks
        nsrc = blocks.null_source(4)
        nsnk = blocks.null_sink(4)
        b = blocks.stream_to_vector_decimator(
            4, 1, 1, 1)  # just a random hier block that exists
        tb = gr.top_block()
        tb.connect(nsrc, b, nsnk)
        tb.set_log_level("debug")
        self.assertEqual(tb.log_level(), "debug")
        self.assertEqual(nsrc.log_level(), "debug")
        self.assertEqual(nsnk.log_level(), "debug")
        self.assertEqual(b.one_in_n.log_level(), "debug")
        tb.set_log_level("critical")
        self.assertEqual(tb.log_level(), "critical")
        self.assertEqual(nsrc.log_level(), "critical")
        self.assertEqual(nsnk.log_level(), "critical")
        self.assertEqual(b.one_in_n.log_level(), "critical")


if __name__ == '__main__':
    gr_unittest.run(test_logger)



#!/usr/bin/env python
#
# Copyright 2004,2007,2010,2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, fft, filter, blocks
import math


def sig_source_f(samp_rate, freq, amp, N):
    t = [float(x) / samp_rate for x in range(N)]
    y = [math.sin(2. * math.pi * freq * x) for x in t]
    return y


def fir_filter(x, taps):
    y = []
    x2 = (len(taps) - 1) * [0, ] + x
    delay = (len(taps) - 1) // 2
    for i in range(len(x)):
        yi = 0
        for j in range(len(taps)):
            yi += taps[len(taps) - 1 - j] * x2[i + j]
        y.append(complex(x2[i + delay], yi))
    return y


class test_hilbert(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_hilbert(self):
        tb = self.tb
        ntaps = 51
        sampling_freq = 100

        N = int(ntaps + sampling_freq * 0.10)
        data = sig_source_f(sampling_freq, sampling_freq * 0.10, 1.0, N)
        src1 = blocks.vector_source_f(data)

        taps = filter.firdes.hilbert(ntaps, fft.window.WIN_HAMMING)
        expected_result = fir_filter(data, taps)

        hilb = filter.hilbert_fc(ntaps)
        dst1 = blocks.vector_sink_c()
        tb.connect(src1, hilb)
        tb.connect(hilb, dst1)
        tb.run()
        dst_data = dst1.data()

        self.assertComplexTuplesAlmostEqual(expected_result, dst_data, 5)


if __name__ == '__main__':
    gr_unittest.run(test_hilbert)



#!/usr/bin/env python
#
# Copyright 2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

import time
import pmt
from gnuradio import gr, gr_unittest, blocks


class test_throttle(gr_unittest.TestCase):
    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_throttling(self):
        src_data = list(range(200))
        src = blocks.vector_source_c(src_data)
        thr = blocks.throttle(gr.sizeof_gr_complex, 100)
        dst = blocks.vector_sink_c()
        self.tb.connect(src, thr, dst)

        start_time = time.perf_counter()
        self.tb.run()
        end_time = time.perf_counter()

        total_time = end_time - start_time
        self.assertGreater(total_time, 1.99)
        self.assertLess(total_time, 3.0)

        dst_data = dst.data()
        self.assertEqual(src_data, dst_data)

    def test_rx_rate_tag(self):
        src_data = list(range(400))
        tag = gr.tag_t()
        tag.key = pmt.string_to_symbol("rx_rate")
        tag.value = pmt.to_pmt(200)
        tag.offset = 0

        src = blocks.vector_source_c(src_data, tags=(tag,))
        thr = blocks.throttle(gr.sizeof_gr_complex, 100, ignore_tags=False)
        dst = blocks.vector_sink_c()
        self.tb.connect(src, thr, dst)

        start_time = time.perf_counter()
        self.tb.run()
        end_time = time.perf_counter()

        total_time = end_time - start_time
        self.assertGreater(total_time, 1.99)
        self.assertLess(total_time, 3.0)

        dst_data = dst.data()
        self.assertEqual(src_data, dst_data)

    def test_limited_chunk_throttling(self):
        src_data = [1, 2, 3]
        rate = 10
        chunksize = 10
        src = blocks.vector_source_c(src_data, repeat=True)

        thr = blocks.throttle(gr.sizeof_gr_complex,
                              rate,
                              ignore_tags=True,
                              maximum_items_per_chunk=chunksize)

        dst = blocks.vector_sink_c()
        self.tb.connect(src, thr, dst)

        total_time = 2.0  # seconds
        start_time = time.perf_counter()
        previous_len = 0

        self.tb.start()

        while time.perf_counter() < start_time + total_time:
            time.sleep(0.1)
            current_len = len(dst.data())
            self.assertLessEqual(current_len, previous_len + chunksize)
            previous_len = current_len

        self.tb.stop()

        self.assertGreaterEqual(current_len, chunksize)


if __name__ == '__main__':
    gr_unittest.run(test_throttle)



#!/usr/bin/env python
#
# Copyright 2024 Marcus Müller
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest
import os


class test_prefs (gr_unittest.TestCase):

    def _test_env(self, env: str, subdir: [str, None] = None):
        print(gr.paths.userconf())
        import tempfile
        oldenv = os.getenv(env, "")
        tmpdir = tempfile.mkdtemp()
        targetdir = tmpdir
        if subdir:
            targetdir = os.path.join(tmpdir, subdir)
            os.mkdir(targetdir)
        os.environ[env] = targetdir
        self.assertEqual(str(gr.paths.userconf()), targetdir)
        if subdir:
            os.rmdir(targetdir)
        os.rmdir(tmpdir)
        if oldenv:
            os.putenv(env, oldenv)
        else:
            os.unsetenv(env)

    def test_001_members(self):
        self.assertTrue(gr.paths.tmp())
        self.assertTrue(gr.paths.appdata())
        self.assertTrue(gr.paths.userconf())
        self.assertTrue(gr.paths.persistent())

    def test_002_react_to_GR_PREFS_PATH(self):
        self._test_env("GR_PREFS_PATH")


if __name__ == '__main__':
    gr_unittest.run(test_prefs)



import argparse
import os
from gnuradio.bindtool import BindingGenerator
import pathlib
import tempfile

parser = argparse.ArgumentParser(description='Bind a GR In-Tree Module')
parser.add_argument('names', type=str, nargs='+',
                    help='Names of gr modules to bind (e.g. fft digital analog)')

parser.add_argument('--output_dir', default=tempfile.gettempdir(),
                    help='Output directory of generated bindings')
parser.add_argument('--prefix', help='Prefix of Installed GNU Radio')
parser.add_argument('--src', help='Directory of gnuradio source tree',
                    default=os.path.dirname(os.path.abspath(__file__)) + '/../../..')
parser.add_argument(
    '--include', help='Additional Include Dirs, comma separated', default='')
args = parser.parse_args()

# NOTES:
# To generate QTGUI requires adding the QT headers as --include, e.g.
# bind_gr_module.py --prefix /share/gnuradio/grpybind --output_dir /share/tmp/take5
#   --include /usr/include/x86_64-linux-gnu/qt5/QtGui,/usr/include/x86_64-linux-gnu/qt5,
#   /usr/include/x86_64-linux-gnu/qt5/QtCore,/usr/include/x86_64-linux-gnu/qt5/QtWidgets,/usr/include/qwt qtgui

# To generate UHD requires adding the UHD headers, e.g.
# python3 /share/gnuradio/grpybind/src/gnuradio/gr-utils/python/bindtool/scripts/bind_gr_module.py
#   --prefix /share/gnuradio/grpybind
#   --include $UHD_PATH,$UHD_PATH/utils,$UHD_PATH/types,$UHD_PATH/transport,$UHD_PATH/usrp_clock,$UHD_PATH/rfnoc
#   --output_dir /share/tmp/take5 uhd


print(pathlib.Path(__file__).parent.absolute())
print(args)

prefix = args.prefix
output_dir = args.output_dir
includes = args.include
for name in args.names:
    if name not in ['gr', 'pmt']:
        namespace = ['gr', name.replace("-", "_")]
        module_dir = os.path.abspath(
            os.path.join(args.src, 'gr-' + name, 'include'))
        prefix_include_root = 'gnuradio/' + name  # pmt, gnuradio/digital, etc.
    else:
        namespace = [name]
        module_dir = os.path.abspath(os.path.join(
            args.src, 'gnuradio-runtime', 'include'))
        if name == 'gr':
            prefix_include_root = 'gnuradio'
            module_dir = os.path.join(module_dir, 'gnuradio')
        elif name == 'pmt':
            prefix_include_root = 'pmt'
            module_dir = os.path.join(module_dir, 'pmt')

    import warnings
    with warnings.catch_warnings():
        warnings.filterwarnings("ignore", category=DeprecationWarning)
        bg = BindingGenerator(prefix, namespace, prefix_include_root,
                              output_dir, addl_includes=includes, match_include_structure=True, write_json_output=True)
        bg.gen_bindings(module_dir)



#!/usr/bin/env python
#
# Copyright 2004,2007,2010,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, digital, blocks


class test_diff_phasor(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_diff_phasor_cc(self):
        src_data = (0 + 0j, 1 + 0j, -1 + 0j, 3 + 4j, -3 - 4j, -3 + 4j)
        expected_result = (0 + 0j, 0 + 0j, -1 + 0j, -
                           3 - 4j, -25 + 0j, -7 - 24j)
        src = blocks.vector_source_c(src_data)
        op = digital.diff_phasor_cc()
        dst = blocks.vector_sink_c()
        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()               # run the graph and wait for it to finish
        actual_result = dst.data()  # fetch the contents of the sink
        self.assertComplexTuplesAlmostEqual(expected_result, actual_result)


if __name__ == '__main__':
    gr_unittest.run(test_diff_phasor)



#!/usr/bin/env python
#
# Copyright 2007,2010 Free Software Foundation, Inc.
# Copyright 2021 Marcus Müller
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest
import pmt


class test_tag_utils (gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001(self):
        t = gr.tag_t()
        t.offset = 10
        t.key = pmt.string_to_symbol('key')
        t.value = pmt.from_long(23)
        t.srcid = pmt.from_bool(False)
        pt = gr.tag_to_python(t)
        self.assertEqual(pt.key, 'key')
        self.assertEqual(pt.value, 23)
        self.assertEqual(pt.offset, 10)

    def test_comparison(self):
        t = gr.tag_t()
        t.offset = 10
        t.key = pmt.string_to_symbol('key')
        t.value = pmt.from_long(23)
        t.srcid = pmt.from_bool(False)

        t2 = gr.tag_t()
        t2.offset = 100
        t2.key = pmt.string_to_symbol('aaa')
        t2.value = pmt.from_long(230)
        t2.srcid = pmt.from_bool(True)
        self.assertTrue(t < t2)
        self.assertTrue(t == t)
        self.assertTrue(t != t2)
        self.assertFalse(t > t2)
        self.assertFalse(t < t)

    def test_002(self):
        offset = 10
        key = pmt.string_to_symbol('key')
        value = pmt.from_long(23)
        srcid = pmt.from_bool(False)

        format_dict = {'offset': offset,
                       'key': key,
                       'value': value,
                       'srcid': srcid}
        format_list = [offset, key, value, srcid]
        format_tuple = (offset, key, value, srcid)

        t_dict = gr.python_to_tag(format_dict)
        t_list = gr.python_to_tag(format_list)
        t_tuple = gr.python_to_tag(format_tuple)

        self.assertTrue(pmt.equal(t_dict.key, key))
        self.assertTrue(pmt.equal(t_dict.value, value))
        self.assertEqual(t_dict.offset, offset)

        self.assertTrue(pmt.equal(t_list.key, key))
        self.assertTrue(pmt.equal(t_list.value, value))
        self.assertEqual(t_list.offset, offset)

        self.assertTrue(pmt.equal(t_tuple.key, key))
        self.assertTrue(pmt.equal(t_tuple.value, value))
        self.assertEqual(t_tuple.offset, offset)

    def test_003(self):
        offsets = (6, 3, 8)
        key = pmt.string_to_symbol('key')
        srcid = pmt.string_to_symbol('qa_tag_utils')
        tags = []

        for k in offsets:
            t = gr.tag_t()
            t.offset = k
            t.key = key
            t.value = pmt.from_long(k)
            t.srcid = srcid
            tags.append(t)

        for k, t in zip(sorted(offsets), sorted(tags)):
            self.assertEqual(t.offset, k)
            self.assertTrue(pmt.equal(t.key, key))
            self.assertTrue(pmt.equal(t.value, pmt.from_long(k)))
            self.assertTrue(pmt.equal(t.srcid, srcid))

        tmin = min(tags)
        self.assertEqual(tmin.offset, min(offsets))
        self.assertTrue(pmt.equal(tmin.key, key))
        self.assertTrue(pmt.equal(tmin.value, pmt.from_long(min(offsets))))
        self.assertTrue(pmt.equal(tmin.srcid, srcid))

        tmax = max(tags)
        self.assertEqual(tmax.offset, max(offsets))
        self.assertTrue(pmt.equal(tmax.key, key))
        self.assertTrue(pmt.equal(tmax.value, pmt.from_long(max(offsets))))
        self.assertTrue(pmt.equal(tmax.srcid, srcid))


if __name__ == '__main__':
    gr_unittest.run(test_tag_utils)



#!/usr/bin/env python
#
# Copyright 2015 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

# turn this folder into a Python module

from . import channel_construction as cc
from .channel_construction_bec import bhattacharyya_bounds
from .helper_functions import is_power_of_two


CHANNEL_TYPE_AWGN = 'AWGN'
CHANNEL_TYPE_BEC = 'BEC'


def get_z_params(is_prototype, channel, block_size, design_snr, mu):
    print('POLAR code channel construction called with parameters channel={0}, blocksize={1}, design SNR={2}, mu={3}'.format(
        channel, block_size, design_snr, mu))
    if not (channel == 'AWGN' or channel == 'BEC'):
        raise ValueError(
            "channel is {0}, but only BEC and AWGN are supported!".format(channel))
    if not is_power_of_two(block_size):
        raise ValueError(
            "block size={0} is not a power of 2!".format(block_size))
    if design_snr < -1.5917:
        raise ValueError(
            "design SNR={0} < -1.5917. MUST be greater!".format(design_snr))
    if not mu > 0:
        raise ValueError("mu={0} < 1. MUST be > 1!".format(mu))
    if not is_prototype and channel == 'AWGN':
        z_params = cc.load_z_parameters(block_size, design_snr, mu)
        print('Read Z-parameter file: {0}'.format(cc.default_dir() +
              cc.generate_filename(block_size, design_snr, mu)))
        return z_params
    return bhattacharyya_bounds(design_snr, block_size)


def load_frozen_bits_info(is_prototype, channel, block_size, num_info_bits, design_snr, mu):
    num_frozen_bits = block_size - num_info_bits
    if not mu > 0:
        mu = 2
    z_params = get_z_params(is_prototype, channel, block_size, design_snr, mu)
    data_set = {
        'positions': cc.get_frozen_bit_indices_from_z_parameters(z_params, num_frozen_bits),
        'values': [0, ] * num_frozen_bits,
        'block_size': block_size,
        'num_info_bits': num_info_bits,
        'num_frozenbits': num_frozen_bits,
        'design_snr': design_snr,
        'channel': channel,
        'mu': mu,
    }
    return data_set



import warnings
import argparse
from gnuradio.bindtool import BindingGenerator
import sys
import tempfile

parser = argparse.ArgumentParser(description='Bind a GR In-Tree Module')
parser.add_argument('--module', type=str,
                    help='Name of gr module containing file to bind (e.g. fft digital analog)')

parser.add_argument('--output_dir', default=tempfile.gettempdir(),
                    help='Output directory of generated bindings')
parser.add_argument('--prefix', help='Prefix of Installed GNU Radio')

parser.add_argument(
    '--filename', help="File to be parsed")

parser.add_argument(
    '--include', help='Additional Include Dirs, comma separated', default=(), nargs='*')

parser.add_argument(
    '--status', help='Location of output file for general status (used during cmake)', default=None
)
parser.add_argument(
    '--flag_automatic', default='0'
)
parser.add_argument(
    '--flag_pygccxml', default='0'
)

args = parser.parse_args()

prefix = args.prefix
output_dir = args.output_dir
includes = args.include
name = args.module

if name not in ['gr', 'pmt']:
    namespace = ['gr', name]
    prefix_include_root = 'gnuradio/' + name  # pmt, gnuradio/digital, etc.
else:
    namespace = [name]
    if name == 'gr':
        prefix_include_root = 'gnuradio'
    elif name == 'pmt':
        prefix_include_root = 'pmt'

with warnings.catch_warnings():
    warnings.filterwarnings("ignore", category=DeprecationWarning)

    bg = BindingGenerator(args.prefix, namespace,
                          prefix_include_root, output_dir, addl_includes=','.join(args.include), catch_exceptions=False, write_json_output=False, status_output=args.status,
                          flag_automatic=True if args.flag_automatic.lower() in [
                              '1', 'true'] else False,
                          flag_pygccxml=True if args.flag_pygccxml.lower() in ['1', 'true'] else False)
    bg.gen_file_binding(args.filename)



#!/usr/bin/env python
#
# Copyright 2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks
import math


def sig_source_f(samp_rate, freq, amp, N):
    t = [float(x) / samp_rate for x in range(N)]
    y = [amp * math.cos(2. * math.pi * freq * x) for x in t]
    return y


def sig_source_c(samp_rate, freq, amp, N):
    t = [float(x) / samp_rate for x in range(N)]
    y = [math.cos(2. * math.pi * freq * x) +
         1j * math.sin(2. * math.pi * freq * x) for x in t]
    return y


class test_vco(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001(self):
        src_data = 200 * [0, ] + 200 * [0.5, ] + 200 * [1, ]
        expected_result = 200 * [1, ] + \
            sig_source_f(1, 0.125, 1, 200) + \
            sig_source_f(1, 0.25, 1, 200)

        src = blocks.vector_source_f(src_data)
        op = blocks.vco_f(1, math.pi / 2.0, 1)
        dst = blocks.vector_sink_f()

        self.tb.connect(src, op, dst)
        self.tb.run()

        result_data = dst.data()
        self.assertFloatTuplesAlmostEqual(expected_result, result_data, 5)

    def test_002(self):
        src_data = 200 * [0, ] + 200 * [0.5, ] + 200 * [1, ]
        expected_result = 200 * [1, ] + \
            sig_source_c(1, 0.125, 1, 200) + \
            sig_source_c(1, 0.25, 1, 200)

        src = blocks.vector_source_f(src_data)
        op = blocks.vco_c(1, math.pi / 2.0, 1)
        dst = blocks.vector_sink_c()

        self.tb.connect(src, op, dst)
        self.tb.run()

        result_data = dst.data()
        self.assertComplexTuplesAlmostEqual(expected_result, result_data, 5)


if __name__ == '__main__':
    gr_unittest.run(test_vco)



#!/usr/bin/env python
#
# Copyright 2004,2005,2007,2010,2013 Free Software Foundation, Inc.
# Copyright 2024 Skandalis Georgios
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks
import pmt


class test_mute(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_ff_mute(self):
        src_arr = [1.0 for i in range(50)]
        src = blocks.vector_source_f(src_arr, False)
        dst = blocks.vector_sink_f()
        mute = blocks.mute_ff(False)

        self.tb.connect(src, mute, dst)

        mute._post(pmt.intern("set_mute"), pmt.from_bool(True))

        self.tb.run()
        self.tb.stop()
        self.tb.wait()

        self.assertFalse(1.0 in dst.data())

    def test_ff_unmute(self):
        src_arr = [1.0 for i in range(50)]
        src = blocks.vector_source_f(src_arr, False)
        dst = blocks.vector_sink_f()
        mute = blocks.mute_ff(False)

        self.tb.connect(src, mute, dst)

        mute._post(pmt.intern("set_mute"), pmt.from_bool(False))

        self.tb.run()
        self.tb.stop()
        self.tb.wait()

        self.assertFalse(0.0 in dst.data())

    def test_ff_mute_pdu(self):
        src_arr = [1.0 for i in range(50)]
        src = blocks.vector_source_f(src_arr, False)
        dst = blocks.vector_sink_f()
        mute = blocks.mute_ff(False)

        self.tb.connect(src, mute, dst)

        mute._post(pmt.intern("set_mute"), pmt.cons(pmt.PMT_NIL, pmt.from_bool(True)))

        self.tb.run()
        self.tb.stop()
        self.tb.wait()

        self.assertFalse(1.0 in dst.data())

    def test_ff_unmute_pdu(self):
        src_arr = [1.0 for i in range(50)]
        src = blocks.vector_source_f(src_arr, False)
        dst = blocks.vector_sink_f()
        mute = blocks.mute_ff(True)

        self.tb.connect(src, mute, dst)

        mute._post(pmt.intern("set_mute"), pmt.cons(pmt.PMT_NIL, pmt.from_bool(False)))

        self.tb.run()
        self.tb.stop()
        self.tb.wait()

        self.assertFalse(0.0 in dst.data())

    def help_ii(self, src_data, exp_data, op):
        for s in zip(list(range(len(src_data))), src_data):
            src = blocks.vector_source_i(s[1])
            self.tb.connect(src, (op, s[0]))
        dst = blocks.vector_sink_i()
        self.tb.connect(op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(exp_data, result_data)

    def help_ss(self, src_data, exp_data, op):
        for s in zip(list(range(len(src_data))), src_data):
            src = blocks.vector_source_s(s[1])
            self.tb.connect(src, (op, s[0]))
        dst = blocks.vector_sink_s()
        self.tb.connect(op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(exp_data, result_data)

    def help_cc(self, src_data, exp_data, op):
        for s in zip(list(range(len(src_data))), src_data):
            src = blocks.vector_source_c(s[1])
            self.tb.connect(src, (op, s[0]))
        dst = blocks.vector_sink_c()
        self.tb.connect(op, dst)
        self.tb.run()
        result_data = dst.data()
        self.assertEqual(exp_data, result_data)

    def test_unmute_ii(self):
        src_data = [1, 2, 3, 4, 5]
        expected_result = [1, 2, 3, 4, 5]
        op = blocks.mute_ii(False)
        self.help_ii((src_data,), expected_result, op)

    def test_mute_ii(self):
        src_data = [1, 2, 3, 4, 5]
        expected_result = [0, 0, 0, 0, 0]
        op = blocks.mute_ii(True)
        self.help_ii((src_data,), expected_result, op)

    def test_unmute_cc(self):
        src_data = [1, 2, 3, 4, 5]
        expected_result = [1, 2, 3, 4, 5]
        op = blocks.mute_ss(False)
        self.help_ss((src_data,), expected_result, op)

    def test_mute_cc(self):
        src_data = [1, 2, 3, 4, 5]
        expected_result = [1, 2, 3, 4, 5]
        op = blocks.mute_ss(True)
        self.help_ss((src_data,), expected_result, op)

    def test_unmute_cc(self):
        src_data = [1 + 5j, 2 + 5j, 3 + 5j, 4 + 5j, 5 + 5j]
        expected_result = [1 + 5j, 2 + 5j, 3 + 5j, 4 + 5j, 5 + 5j]
        op = blocks.mute_cc(False)
        self.help_cc((src_data,), expected_result, op)

    def test_mute_cc(self):
        src_data = [1 + 5j, 2 + 5j, 3 + 5j, 4 + 5j, 5 + 5j]
        expected_result = [0 + 0j, 0 + 0j, 0 + 0j, 0 + 0j, 0 + 0j]
        op = blocks.mute_cc(True)
        self.help_cc((src_data,), expected_result, op)


if __name__ == '__main__':
    gr_unittest.run(test_mute)



#
# Copyright 2019 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#
""" unittest for gr-blocktool api """


import os
import unittest
import warnings

try:
    import pygccxml
    SKIP_BLOCK_TEST = False
except:
    SKIP_BLOCK_TEST = True

try:
    import apt
    CACHE = apt.cache.Cache()
    CACHE.open()

    PKG = CACHE['castxml']
    if PKG.is_installed:
        SKIP_BLOCK_TEST = False
    else:
        SKIP_BLOCK_TEST = True
except:
    SKIP_BLOCK_TEST = True

from jsonschema import validate

from blocktool import BlockHeaderParser
from blocktool.core.base import BlockToolException
from blocktool.core import Constants
from blocktool import RESULT_SCHEMA


class TestBlocktoolCore(unittest.TestCase):
    """ The Tests for blocktool core """

    def __init__(self, *args, **kwargs):
        super(TestBlocktoolCore, self).__init__(*args, **kwargs)
        self.module = os.path.abspath(os.path.join(os.path.dirname(__file__),
                                                   '../../../../gr-analog'))
        self.test_dir = os.path.abspath(os.path.join(self.module,
                                                     'include/gnuradio/analog'))

    def is_int(self, number):
        """
        Check for int conversion
        """
        try:
            int(number)
            return True
        except ValueError:
            return False

    @classmethod
    def setUpClass(cls):
        """ create a temporary Blocktool object """
        try:
            warnings.simplefilter("ignore", ResourceWarning)
        except NameError:
            pass
        test_path = {}
        target_file = os.path.abspath(os.path.join(os.path.dirname(
            __file__), '../../../../gr-analog/include/gnuradio/analog', 'agc2_cc.h'))
        test_path['file_path'] = target_file
        cls.test_obj = BlockHeaderParser(**test_path).get_header_info()

    @unittest.skipIf(SKIP_BLOCK_TEST, 'pygccxml not found, skipping this unittest')
    def test_blocktool_exceptions(self):
        """
        tests for blocktool exceptions
        """
        # test for non-existent header or invalid headers
        test_dict = {}
        test_dict['file_path'] = os.path.abspath(
            os.path.join(self.test_dir, 'sample.h'))
        with self.assertRaises(BlockToolException):
            BlockHeaderParser(**test_dict).run_blocktool()
        # test for invalid header file
        test_dict['file_path'] = os.path.abspath(
            os.path.join(self.test_dir, 'CMakeLists.txt'))
        if not os.path.basename(test_dict['file_path']).endswith('.h'):
            with self.assertRaises(BlockToolException):
                BlockHeaderParser(**test_dict).run_blocktool()

    @unittest.skipIf(SKIP_BLOCK_TEST, 'pygccxml not found, skipping this unittest')
    def test_namespace(self):
        """ test for header namespace """
        module_name = os.path.basename(self.module)
        self.assertEqual(self.test_obj['namespace'][0], 'gr')
        self.assertEqual(self.test_obj['namespace'][1], module_name.split('-')[-1])

    @unittest.skipIf(SKIP_BLOCK_TEST, 'pygccxml not found, skipping this unittest')
    def test_io_signature(self):
        """ test for io_signature """
        input_signature = self.test_obj['io_signature']['input']['signature']
        output_signature = self.test_obj['io_signature']['output']['signature']
        valid_signature = False
        if input_signature and output_signature in Constants.SIGNATURE_LIST:
            valid_signature = True
        self.assertTrue(valid_signature)
        valid_io_stream = False
        input_max = self.test_obj['io_signature']['input']['max_streams']
        input_min = self.test_obj['io_signature']['input']['min_streams']
        output_max = self.test_obj['io_signature']['output']['max_streams']
        output_min = self.test_obj['io_signature']['output']['min_streams']
        if self.is_int(input_max) and self.is_int(input_min) and self.is_int(output_max) and self.is_int(output_min):
            valid_io_stream = True
        self.assertTrue(valid_io_stream)

    @unittest.skipIf(SKIP_BLOCK_TEST, 'pygccxml not found, skipping this unittest')
    def test_message_port(self):
        """ test for message ports """
        input_port = self.test_obj['message_port']['input']
        output_port = self.test_obj['message_port']['output']
        valid_input_message_port = True
        valid_output_message_port = True
        if input_port:
            for port in input_port:
                if not port['id']:
                    valid_input_message_port = False
        if output_port:
            for port in output_port:
                if not port['id']:
                    valid_output_message_port = False
        self.assertTrue(valid_input_message_port)
        self.assertTrue(valid_output_message_port)

    @unittest.skipIf(SKIP_BLOCK_TEST, 'pygccxml not found, skipping this unittest')
    def test_factory_signature(self):
        """ test for factory signature in the header """
        valid_factory_arg = True
        if self.test_obj['make']['arguments']:
            for arguments in self.test_obj['make']['arguments']:
                if not arguments['name'] or not arguments['dtype']:
                    valid_factory_arg = False
        self.assertTrue(valid_factory_arg)

    @unittest.skipIf(SKIP_BLOCK_TEST, 'pygccxml not found, skipping this unittest')
    def test_methods(self):
        """ test for methods """
        valid_method = True
        if self.test_obj['methods']:
            for arguments in self.test_obj['methods']:
                if not arguments['name']:
                    valid_method = False
                if arguments['arguments_type']:
                    for args in arguments['arguments_type']:
                        if not args['name'] or not args['dtype']:
                            valid_method = False
        self.assertTrue(valid_method)

    @unittest.skipIf(SKIP_BLOCK_TEST, 'pygccxml not found, skipping this unittest')
    def test_properties(self):
        """ test for properties """
        valid_properties = True
        if self.test_obj['properties']:
            for arguments in self.test_obj['properties']:
                if not arguments['name'] or not arguments['dtype']:
                    valid_properties = False
        self.assertTrue(valid_properties)

    @unittest.skipIf(SKIP_BLOCK_TEST, 'pygccxml not found, skipping this unittest')
    def test_result_format(self):
        """ test for parsed blocktool output format """
        valid_schema = False
        try:
            validate(instance=self.test_obj, schema=RESULT_SCHEMA)
            valid_schema = True
        except BlockToolException:
            raise BlockToolException
        self.assertTrue(valid_schema)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#
# Copyright 2020 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

from gnuradio import gr, gr_unittest
from gnuradio import fec
from gnuradio import blocks, analog, pdu

# This test tries to checks for the kind of buffer overflows in the
# FECAPI convolutional decoder that were fixed in #2965


class test_fecapi_cc_buffer_overflow(gr_unittest.TestCase):

    def run_with_frame_len(self, frame_len):
        self.tb = gr.top_block()
        noise = analog.noise_source_f(analog.GR_GAUSSIAN, 1, 0)
        head = blocks.head(gr.sizeof_float, self.n_frames * frame_len)
        tag = blocks.stream_to_tagged_stream(
            gr.sizeof_float, 1, frame_len, "packet_len")
        stream2pdu = pdu.tagged_stream_to_pdu(gr.types.float_t, 'packet_len')

        viterbi = fec.cc_decoder.make(
            frame_len // 2, 7, 2, [79, -109], 0, -1, fec.CC_TERMINATED, False)
        cc_decoder = fec.async_decoder(viterbi, False, False, frame_len)

        self.tb.connect(noise, head, tag, stream2pdu)
        self.tb.msg_connect((stream2pdu, 'pdus'), (cc_decoder, 'in'))

        self.tb.run()
        # the test is considered successful if it hasn't segfaulted or aborted

        self.tb = None

    def test_buffer_overflow_00(self):
        self.n_frames = 10

        for frame_len in range(1000, 2000):
            with self.subTest(frame_length=frame_len):
                self.run_with_frame_len(frame_len)


if __name__ == '__main__':
    gr_unittest.run(test_fecapi_cc_buffer_overflow)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2018-2021 National Technology & Engineering Solutions of Sandia, LLC
# (NTESS). Under the terms of Contract DE-NA0003525 with NTESS, the U.S. Government
# retains certain rights in this software.
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gnuradio import gr, gr_unittest, blocks, pdu
import numpy as np
import pmt
import time


class qa_tags_to_pdu (gr_unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_001_simple(self):
        self.tb = gr.top_block()
        start_time = 0.1
        sob_tag = gr.tag_utils.python_to_tag(
            (34, pmt.intern("SOB"), pmt.PMT_T, pmt.intern("src")))
        eob_tag = gr.tag_utils.python_to_tag(
            (34 + (8 * 31), pmt.intern("EOB"), pmt.PMT_T, pmt.intern("src")))
        vs = blocks.vector_source_s(range(350), False, 1, [sob_tag, eob_tag])
        t2p = pdu.tags_to_pdu_s(pmt.intern('SOB'), pmt.intern(
            'EOB'), 1024, 512000, ([]), False, 0, start_time)
        t2p.set_eob_parameters(8, 0)
        dbg = blocks.message_debug()
        self.tb.connect(vs, t2p)
        self.tb.msg_connect((t2p, 'pdus'), (dbg, 'store'))
        expected_vec = pmt.init_s16vector((8 * 31), range(34, 34 + (8 * 31)))
        expected_time = start_time + (34 / 512000.0)

        self.tb.run()

        self.assertEqual(dbg.num_messages(), 1)
        self.assertTrue(pmt.equal(pmt.cdr(dbg.get_message(0)), expected_vec))
        time_tuple1 = pmt.dict_ref(
            pmt.car(dbg.get_message(0)), pmt.intern("rx_time"), pmt.PMT_NIL)
        self.assertAlmostEqual(pmt.to_uint64(pmt.tuple_ref(
            time_tuple1, 0)) + pmt.to_double(pmt.tuple_ref(time_tuple1, 1)), expected_time)

        self.tb = None

    def test_002_secondSOB(self):
        self.tb = gr.top_block()
        start_time = 4.999999999
        sob_tag = gr.tag_utils.python_to_tag(
            (34, pmt.intern("SOB"), pmt.PMT_T, pmt.intern("src")))
        sob_tag2 = gr.tag_utils.python_to_tag(
            (51, pmt.intern("SOB"), pmt.PMT_T, pmt.intern("src")))
        eob_tag = gr.tag_utils.python_to_tag(
            (51 + (8 * 26), pmt.intern("EOB"), pmt.PMT_T, pmt.intern("src")))
        vs = blocks.vector_source_s(range(350), False, 1, [
                                    sob_tag, sob_tag2, eob_tag])
        t2p = pdu.tags_to_pdu_s(pmt.intern('SOB'), pmt.intern(
            'EOB'), 1024, 460800, ([]), False, 0, start_time)
        t2p.set_eob_parameters(8, 0)
        dbg = blocks.message_debug()
        self.tb.connect(vs, t2p)
        self.tb.msg_connect((t2p, 'pdus'), (dbg, 'store'))
        expected_vec = pmt.init_s16vector((8 * 26), range(51, 51 + (8 * 26)))
        expected_time = start_time + (51 / 460800.0)

        self.tb.run()

        self.assertEqual(dbg.num_messages(), 1)
        self.assertTrue(pmt.equal(pmt.cdr(dbg.get_message(0)), expected_vec))
        time_tuple1 = pmt.dict_ref(
            pmt.car(dbg.get_message(0)), pmt.intern("rx_time"), pmt.PMT_NIL)
        self.assertAlmostEqual(pmt.to_uint64(pmt.tuple_ref(
            time_tuple1, 0)) + pmt.to_double(pmt.tuple_ref(time_tuple1, 1)), expected_time)

        self.tb = None

    def test_003_double_eob_rej_tt_update(self):
        self.tb = gr.top_block()
        start_time = 0.0
        sob_tag = gr.tag_utils.python_to_tag(
            (51, pmt.intern("SOB"), pmt.PMT_T, pmt.intern("src")))
        eob_tag = gr.tag_utils.python_to_tag(
            (51 + (8 * 11), pmt.intern("EOB"), pmt.PMT_T, pmt.intern("src")))
        time_tuple = pmt.make_tuple(pmt.from_uint64(4), pmt.from_double(
            0.125), pmt.from_uint64(10000000), pmt.from_double(4000000.0))
        time_tag = gr.tag_utils.python_to_tag(
            (360, pmt.intern("rx_time"), time_tuple, pmt.intern("src")))
        sob_tag2 = gr.tag_utils.python_to_tag(
            (400, pmt.intern("SOB"), pmt.PMT_T, pmt.intern("src")))
        eob_tag2e = gr.tag_utils.python_to_tag(
            (409, pmt.intern("EOB"), pmt.PMT_T, pmt.intern("src")))
        eob_tag2 = gr.tag_utils.python_to_tag(
            (416, pmt.intern("EOB"), pmt.PMT_T, pmt.intern("src")))
        vs = blocks.vector_source_s(range(500), False, 1, [
                                    sob_tag, eob_tag, time_tag, sob_tag2, eob_tag2e, eob_tag2])
        t2p = pdu.tags_to_pdu_s(pmt.intern('SOB'), pmt.intern(
            'EOB'), 1024, 1000000, ([]), False, 0, start_time)
        t2p.set_eob_parameters(8, 0)
        dbg = blocks.message_debug()
        self.tb.connect(vs, t2p)
        self.tb.msg_connect((t2p, 'pdus'), (dbg, 'store'))
        expected_vec1 = pmt.init_s16vector((8 * 11), range(51, 51 + (8 * 11)))
        expected_vec2 = pmt.init_s16vector(16, list(range(400, 409)) + [0] * 7)
        expected_time1 = start_time + (51 / 1000000.0)
        expected_time2 = 4.125 + ((400 - 360) / 1000000.0)

        self.tb.run()

        self.assertEqual(dbg.num_messages(), 2)
        self.assertTrue(pmt.equal(pmt.cdr(dbg.get_message(0)), expected_vec1))
        self.assertTrue(pmt.equal(pmt.cdr(dbg.get_message(1)), expected_vec2))
        time_tuple1 = pmt.dict_ref(
            pmt.car(dbg.get_message(0)), pmt.intern("rx_time"), pmt.PMT_NIL)
        time_tuple2 = pmt.dict_ref(
            pmt.car(dbg.get_message(1)), pmt.intern("rx_time"), pmt.PMT_NIL)
        self.assertAlmostEqual(pmt.to_uint64(pmt.tuple_ref(
            time_tuple1, 0)) + pmt.to_double(pmt.tuple_ref(time_tuple1, 1)), expected_time1)
        self.assertAlmostEqual(pmt.to_uint64(pmt.tuple_ref(
            time_tuple2, 0)) + pmt.to_double(pmt.tuple_ref(time_tuple2, 1)), expected_time2)

        self.tb = None

    def test_004_boost_time(self):
        self.tb = gr.top_block()
        start_time = 0.1
        sob_tag = gr.tag_utils.python_to_tag(
            (34, pmt.intern("SOB"), pmt.PMT_T, pmt.intern("src")))
        eob_tag = gr.tag_utils.python_to_tag(
            (34 + (8 * 31), pmt.intern("EOB"), pmt.PMT_T, pmt.intern("src")))
        vs = blocks.vector_source_s(range(350), False, 1, [sob_tag, eob_tag])
        t2p = pdu.tags_to_pdu_s(pmt.intern('SOB'), pmt.intern(
            'EOB'), 1024, 512000, ([]), False, 0, start_time)
        t2p.enable_time_debug(True)
        t2p.set_eob_parameters(8, 0)
        dbg = blocks.message_debug()

        self.tb.connect(vs, t2p)
        self.tb.msg_connect((t2p, 'pdus'), (dbg, 'store'))
        expected_vec = pmt.init_s16vector((8 * 31), range(34, 34 + (8 * 31)))
        expected_time = start_time + (34 / 512000.0)
        ts = time.time()
        self.tb.run()

        self.assertEqual(dbg.num_messages(), 1)
        self.assertTrue(pmt.equal(pmt.cdr(dbg.get_message(0)), expected_vec))
        time_tuple1 = pmt.dict_ref(
            pmt.car(dbg.get_message(0)), pmt.intern("rx_time"), pmt.PMT_NIL)
        self.assertAlmostEqual(pmt.to_uint64(pmt.tuple_ref(
            time_tuple1, 0)) + pmt.to_double(pmt.tuple_ref(time_tuple1, 1)), expected_time)
        #wct = pmt.to_double(pmt.dict_ref(pmt.car(dbg.get_message(0)), pmt.intern("wall_clock_time"), pmt.PMT_NIL))
        #self.assertLess(wct - ts, 1.0)

        self.tb = None

    def test_005_two_sobs_misaligned(self):
        # Two SOB tags and the SOB-to-EOB length is not aligned
        self.tb = gr.top_block()
        start_time = 0.1
        sob_tag = gr.tag_utils.python_to_tag(
            (34, pmt.intern("SOB"), pmt.PMT_T, pmt.intern("src")))
        sob_tag2 = gr.tag_utils.python_to_tag(
            (35, pmt.intern("SOB"), pmt.PMT_T, pmt.intern("src")))
        eob_tag = gr.tag_utils.python_to_tag(
            (34 + (8 * 31), pmt.intern("EOB"), pmt.PMT_T, pmt.intern("src")))
        vs = blocks.vector_source_s(range(1350), False, 1, [
                                    sob_tag, sob_tag2, eob_tag])
        t2p = pdu.tags_to_pdu_s(pmt.intern('SOB'), pmt.intern(
            'EOB'), 1024, 512000, ([]), False, 0, start_time)
        t2p.set_eob_parameters(8, 0)
        dbg = blocks.message_debug()
        self.tb.connect(vs, t2p)
        self.tb.msg_connect((t2p, 'pdus'), (dbg, 'store'))
        expected_vec = pmt.init_s16vector(
            (8 * 31), list(range(35, 34 + (8 * 31))) + [0])
        expected_time = start_time + (35 / 512000.0)

        self.tb.run()

        self.assertEqual(dbg.num_messages(), 1)
        # print "got ", dbg.get_message(0)
        # print "expected", expected_vec
        # print "len is {}".format(len(pmt.to_python(pmt.cdr(dbg.get_message(0)))))
        self.assertTrue(pmt.equal(pmt.cdr(dbg.get_message(0)), expected_vec))
        time_tuple1 = pmt.dict_ref(
            pmt.car(dbg.get_message(0)), pmt.intern("rx_time"), pmt.PMT_NIL)
        self.assertAlmostEqual(pmt.to_uint64(pmt.tuple_ref(
            time_tuple1, 0)) + pmt.to_double(pmt.tuple_ref(time_tuple1, 1)), expected_time)

        self.tb = None

    def test_006_max_pdu_size(self):
        # two SOB tags exactly max_pdu_size samples apart, with an SOB-to-EOB length that is not divisible by the alignment size
        self.tb = gr.top_block()
        start_time = 0.1
        max_size = 100
        sob_tag = gr.tag_utils.python_to_tag(
            (10, pmt.intern("SOB"), pmt.PMT_T, pmt.intern("src")))
        eob_tag = gr.tag_utils.python_to_tag(
            (91, pmt.intern("EOB"), pmt.PMT_T, pmt.intern("src")))
        sob_tag3 = gr.tag_utils.python_to_tag(
            (11 + max_size, pmt.intern("SOB"), pmt.PMT_T, pmt.intern("src")))
        vs = blocks.vector_source_s(range(1350), False, 1, [
                                    sob_tag, eob_tag, sob_tag3])
        t2p = pdu.tags_to_pdu_s(pmt.intern('SOB'), pmt.intern(
            'EOB'), 1024, 512000, ([]), False, 0, start_time)
        t2p.set_eob_parameters(10, 0)
        t2p.set_max_pdu_size(max_size)

        dbg = blocks.message_debug()
        self.tb.connect(vs, t2p)
        self.tb.msg_connect((t2p, 'pdus'), (dbg, 'store'))
        expected_vec = pmt.init_s16vector(
            (9 * 10), list(range(10, 91)) + [0] * 9)
        expected_time = start_time + (10 / 512000.0)

        self.tb.run()

        # assertions for the first PDU only, second PDU will exist
        self.assertEqual(dbg.num_messages(), 2)
        # print "got ", dbg.get_message(0)
        # print "expected", expected_vec
        # print "len is {}".format(len(pmt.to_python(pmt.cdr(dbg.get_message(0)))))
        self.assertTrue(pmt.equal(pmt.cdr(dbg.get_message(0)), expected_vec))
        time_tuple1 = pmt.dict_ref(
            pmt.car(dbg.get_message(0)), pmt.intern("rx_time"), pmt.PMT_NIL)
        self.assertAlmostEqual(pmt.to_uint64(pmt.tuple_ref(
            time_tuple1, 0)) + pmt.to_double(pmt.tuple_ref(time_tuple1, 1)), expected_time)

        self.tb = None

    def test_007_max_pdu_size_SOBs(self):
        # two SOB tags exactly max_pdu_size samples apart
        self.tb = gr.top_block()
        start_time = 0.1
        max_size = 100
        sob_tag = gr.tag_utils.python_to_tag(
            (10, pmt.intern("SOB"), pmt.PMT_T, pmt.intern("src")))
        sob_tag3 = gr.tag_utils.python_to_tag(
            (10 + max_size, pmt.intern("SOB"), pmt.PMT_T, pmt.intern("src")))

        vs = blocks.vector_source_s(range(1350), False, 1, [sob_tag, sob_tag3])
        t2p = pdu.tags_to_pdu_s(pmt.intern('SOB'), pmt.intern(
            'EOB'), 1024, 512000, ([]), False, 0, start_time)
        t2p.set_eob_parameters(10, 0)
        t2p.set_max_pdu_size(max_size)

        dbg = blocks.message_debug()
        self.tb.connect(vs, t2p)
        self.tb.msg_connect((t2p, 'pdus'), (dbg, 'store'))
        expected_vec = pmt.init_s16vector((max_size), range(10, 10 + max_size))
        expected_time = start_time + (10 / 512000.0)

        self.tb.run()

        # assertions for the first PDU only, second PDU will exist
        self.assertEqual(dbg.num_messages(), 2)
        # print "got ", dbg.get_message(0)
        # print "expected", expected_vec
        self.assertTrue(pmt.equal(pmt.cdr(dbg.get_message(0)), expected_vec))
        time_tuple1 = pmt.dict_ref(
            pmt.car(dbg.get_message(0)), pmt.intern("rx_time"), pmt.PMT_NIL)
        self.assertAlmostEqual(pmt.to_uint64(pmt.tuple_ref(
            time_tuple1, 0)) + pmt.to_double(pmt.tuple_ref(time_tuple1, 1)), expected_time)

        self.tb = None


# TODO: add more tests:
#   - test ability to correctly handle max-length pdus
#   - test callbacks
#   - test message setting interface
#   - test time-tracking tags


if __name__ == '__main__':
    gr_unittest.run(qa_tags_to_pdu)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2020 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

from PyQt5.QtWidgets import QFrame, QVBoxLayout, QLabel
from PyQt5 import Qt
from PyQt5.QtCore import Qt as Qtc
from PyQt5.QtCore import QSize
from gnuradio import gr
import pmt


class LabeledDialControl(QFrame):
    def __init__(self, lbl='', parent=None, minimum=0, maximum=100, defaultvalue=0,
                 backgroundColor='default', changedCallback=None,
                 minsize=100, isFloat=False, scaleFactor=1, showvalue=False,
                 outputmsgname='value'):
        QFrame.__init__(self, parent)
        self.numberControl = DialControl(minimum, maximum, defaultvalue / scaleFactor, backgroundColor,
                                         self.valChanged, changedCallback, minsize)

        layout = QVBoxLayout()

        self.outputmsgname = outputmsgname
        self.showvalue = showvalue
        self.isFloat = isFloat
        self.scaleFactor = scaleFactor
        self.lbl = lbl
        self.lblcontrol = QLabel(lbl, self)
        self.lblcontrol.setAlignment(Qtc.AlignCenter)

        if self.showvalue:
            textstr = self.buildTextStr(defaultvalue)
            self.lblcontrol.setText(textstr)

        if len or self.showvalue:
            self.hasLabel = True
            layout.addWidget(self.lblcontrol)
        else:
            self.hasLabel = False

        layout.addWidget(self.numberControl)

        layout.setAlignment(Qtc.AlignCenter)
        self.setLayout(layout)
        self.show()

    def buildTextStr(self, new_value):
        textstr = ""
        if self.lbl:
            textstr = self.lbl + " - "

        if self.isFloat:
            textstr += "%.2f" % (new_value)
        else:
            textstr += str(new_value)

        return textstr

    def valChanged(self, new_value):
        if not self.showvalue:
            return

        if int(self.scaleFactor) != 1:
            new_value = new_value * self.scaleFactor

        textstr = self.buildTextStr(new_value)
        self.lblcontrol.setText(textstr)


class DialControl(Qt.QDial):
    def __init__(self, minimum=0, maximum=100, defaultvalue=0, backgroundColor='default',
                 lablelCallback=None, changedCallback=None, minsize=100):
        Qt.QDial.__init__(self)

        if backgroundColor != "default":
            self.setStyleSheet("background-color: " + backgroundColor + ";")

        self.minsize = minsize
        self.changedCallback = changedCallback
        self.lablelCallback = lablelCallback
        super().setMinimum(int(minimum))
        super().setMaximum(int(maximum))
        super().setValue(int(defaultvalue))
        super().valueChanged.connect(self.sliderMoved)

    def minimumSizeHint(self):
        return QSize(self.minsize, self.minsize)

    def sliderMoved(self):
        if self.changedCallback is not None:
            self.changedCallback(self.value())

        if self.lablelCallback is not None:
            self.lablelCallback(self.value())


class GrDialControl(gr.sync_block, LabeledDialControl):
    """
    This block creates a dial control. The control does control a
    variable which can be used for other items.  Leave the label
    blank to use the variable id as the label. The block also
    creates an optional message with the control value that
    can be used in message-based applications.

    Note: Dials only produce integer values, so the scale factor
    can be used with the min/max to adjust the output value to
    the desired range.  Think of the min/max as the increments,
    and the scale factor as the adjustment to get the values you want.
    """

    def __init__(self, lbl, parent, minimum, maximum, defaultvalue, backgroundColor='default',
                 varCallback=None, isFloat=False,
                 scaleFactor=1, minsize=100, showvalue=False, outputmsgname='value'):
        gr.sync_block.__init__(self, name="GrDialControl",
                               in_sig=None, out_sig=None)
        LabeledDialControl.__init__(self, lbl, parent, minimum, maximum, defaultvalue,
                                    backgroundColor, self.valueChanged, minsize, isFloat,
                                    scaleFactor, showvalue)

        self.outputmsgname = outputmsgname
        self.varCallback = varCallback
        self.scaleFactor = scaleFactor
        self.isFloat = isFloat
        self.message_port_register_out(pmt.intern("value"))

    def valueChanged(self, new_value):
        if int(self.scaleFactor) != 1:
            new_value = new_value * self.scaleFactor

        if self.varCallback is not None:
            self.varCallback(new_value)

        if self.isFloat:
            self.message_port_pub(pmt.intern("value"), pmt.cons(pmt.intern(self.outputmsgname),
                                                                pmt.from_double(new_value)))
        else:
            self.message_port_pub(pmt.intern("value"), pmt.cons(pmt.intern(self.outputmsgname),
                                                                pmt.from_long(new_value)))



#!/usr/bin/env python
#
# Copyright 2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks
import pmt
import numpy
import time

# Simple block to generate messages


class message_generator(gr.sync_block):
    def __init__(self, msg_list, msg_interval):
        gr.sync_block.__init__(
            self,
            name="message generator",
            in_sig=[numpy.float32],
            out_sig=None
        )
        self.msg_list = msg_list
        self.msg_interval = msg_interval
        self.msg_ctr = 0
        self.message_port_register_out(pmt.intern('out_port'))

    def work(self, input_items, output_items):
        inLen = len(input_items[0])
        while self.msg_ctr < len(self.msg_list) and \
                (self.msg_ctr * self.msg_interval) < \
                (self.nitems_read(0) + inLen):
            self.message_port_pub(pmt.intern('out_port'),
                                  self.msg_list[self.msg_ctr])
            self.msg_ctr += 1
        return inLen

# Simple block to consume messages


class message_consumer(gr.sync_block):
    def __init__(self):
        gr.sync_block.__init__(
            self,
            name="message consumer",
            in_sig=None,
            out_sig=None
        )
        self.msg_list = []
        self.message_port_register_in(pmt.intern('in_port'))
        self.set_msg_handler(pmt.intern('in_port'),
                             self.handle_msg)

    def handle_msg(self, msg):
        # Create a new PMT from long value and put in list
        self.msg_list.append(pmt.from_long(pmt.to_long(msg)))


class test_python_message_passing(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_000(self):
        num_msgs = 10
        msg_interval = 1000
        msg_list = []
        for i in range(num_msgs):
            msg_list.append(pmt.from_long(i))

        # Create vector source with dummy data to trigger messages
        src_data = []
        for i in range(num_msgs * msg_interval):
            src_data.append(float(i))
        src = blocks.vector_source_f(src_data, False)
        msg_gen = message_generator(msg_list, msg_interval)
        msg_cons = message_consumer()

        # Connect vector source to message gen
        self.tb.connect(src, msg_gen)

        # Connect message generator to message consumer
        self.tb.msg_connect(msg_gen, 'out_port', msg_cons, 'in_port')

        # Verify that the message port query functions work
        self.assertEqual(
            pmt.to_python(
                msg_gen.message_ports_out())[0],
            'out_port')
        self.assertEqual(
            'in_port' in pmt.to_python(
                msg_cons.message_ports_in()), True)

        # Run to verify message passing
        self.tb.run()

        # Verify that the message consumer got all the messages
        self.assertEqual(num_msgs, len(msg_cons.msg_list))
        for i in range(num_msgs):
            self.assertTrue(pmt.equal(msg_list[i], msg_cons.msg_list[i]))


if __name__ == '__main__':
    gr_unittest.run(test_python_message_passing)



#!/usr/bin/env python
#
# Copyright 2021 NTESS LLC.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks, pdu
import pmt
import numpy as np
import time


class qa_time_delta(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

        self.time_delta = pdu.time_delta(pmt.intern(
            "sys time delta (ms)"), pmt.intern("system_time"))
        self.debug = blocks.message_debug()

        self.tb.msg_connect((self.time_delta, 'pdu'), (self.debug, 'store'))

    def tearDown(self):
        self.tb = None

    def test_001_invalid_a(self):
        self.tb.start()
        self.time_delta.to_basic_block()._post(
            pmt.intern("pdu"), pmt.intern("NOT A PDU"))
        time.sleep(0.01)  # short delay to ensure the message is processed
        self.tb.stop()
        self.tb.wait()

        # nothing should be produced in this case
        self.assertEqual(0, self.debug.num_messages())

    def test_001_invalid_b(self):
        in_data = [0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1,
                   1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1]
        meta = pmt.dict_add(pmt.make_dict(), pmt.intern(
            'sam'), pmt.from_double(25.1))
        in_pdu = pmt.cons(meta, pmt.init_c32vector(len(in_data), in_data))

        # set up fg
        self.tb.start()
        self.time_delta.to_basic_block()._post(pmt.intern("pdu"), in_pdu)
        time.sleep(0.01)  # short delay to ensure the message is processed
        self.tb.stop()
        self.tb.wait()

        # nothing should be produced in this case
        self.assertEqual(0, self.debug.num_messages())

    def test_002_normal(self):
        tnow = time.time()

        in_data = [0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1,
                   1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1]
        meta = pmt.dict_add(pmt.make_dict(), pmt.intern(
            'system_time'), pmt.from_double(tnow - 10.0))
        in_pdu = pmt.cons(meta, pmt.init_c32vector(len(in_data), in_data))

        # set up fg
        self.tb.start()
        self.time_delta.to_basic_block()._post(pmt.intern("pdu"), in_pdu)
        self.waitFor(lambda: self.debug.num_messages() ==
                     1, timeout=1.0, poll_interval=0.01)
        self.tb.stop()
        self.tb.wait()

        # check data
        self.assertEqual(1, self.debug.num_messages())
        a_meta = pmt.car(self.debug.get_message(0))
        time_tag = pmt.dict_ref(a_meta, pmt.intern("system_time"), pmt.PMT_NIL)
        delta_tag = pmt.dict_ref(a_meta, pmt.intern(
            "sys time delta (ms)"), pmt.PMT_NIL)
        self.assertAlmostEqual(tnow - 10.0, pmt.to_double(time_tag), delta=1e-6)
        self.assertAlmostEqual(10000, pmt.to_double(delta_tag), delta=500)


if __name__ == '__main__':
    gr_unittest.run(qa_time_delta)



from __future__ import absolute_import, print_function

# Standard modules
import logging

import xml.etree.ElementTree as ET

from ast import literal_eval
from qtpy import QtGui, QtCore, QtWidgets
from qtpy.QtCore import Qt

# Custom modules
from .canvas.block import Block
from .. import base
from .canvas.flowgraph import FlowgraphScene, Flowgraph
from .canvas.colors import LIGHT_FLOWGRAPH_BACKGROUND_COLOR, DARK_FLOWGRAPH_BACKGROUND_COLOR

from ...core.generator import Generator

# Logging
log = logging.getLogger(f"grc.application.{__name__}")

DEFAULT_MAX_X = 400
DEFAULT_MAX_Y = 300


class FlowgraphView(
    QtWidgets.QGraphicsView, base.Component
):  # added base.Component so it can see platform
    def __init__(self, parent, platform, filename=None):
        super(FlowgraphView, self).__init__()
        self.setParent(parent)
        self.setAlignment(Qt.AlignLeft | Qt.AlignTop)

        self.setScene(FlowgraphScene(self, platform))

        self.scalefactor = 0.8
        self.scale(self.scalefactor, self.scalefactor)

        self.setSceneRect(0, 0, DEFAULT_MAX_X, DEFAULT_MAX_Y)
        if filename is not None:
            self.readFile(filename)
        else:
            self.initEmpty()

        self.fitInView(self.scene().sceneRect(), QtCore.Qt.KeepAspectRatio)
        if self.app.theme == "dark":
            self.setBackgroundBrush(QtGui.QBrush(DARK_FLOWGRAPH_BACKGROUND_COLOR))
        else:
            self.setBackgroundBrush(QtGui.QBrush(LIGHT_FLOWGRAPH_BACKGROUND_COLOR))

        self.isPanning = False
        self.mousePressed = False

        self.setDragMode(self.RubberBandDrag)

        self.generator = None
        self.process = None

    def createActions(self, actions):
        log.debug("Creating actions")

    def contextMenuEvent(self, event):
        super(FlowgraphView, self).contextMenuEvent(event)

    def createMenus(self, actions, menus):
        log.debug("Creating menus")

    def createToolbars(self, actions, toolbars):
        log.debug("Creating toolbars")

    def get_generator(self) -> Generator:
        return self.generator

    def process_is_done(self) -> bool:
        if self.process is None:
            return True
        else:
            return (False if self.process.returncode is None else True)

    def readFile(self, filename):
        tree = ET.parse(filename)

        for xml_block in tree.findall("block"):
            attrib = {}
            params = []
            block_key = xml_block.find("key").text

            for param in xml_block.findall("param"):
                key = param.find("key").text
                value = param.find("value").text
                if key.startswith("_"):
                    attrib[key] = literal_eval(value)
                else:
                    params.append((key, value))

            # Find block in tree so that we can pull out label
            try:
                block = self.platform.blocks[block_key]

                new_block = Block(block_key, block.label, attrib, params)
                self.scene.addItem(new_block)
                self.newElement.emit(new_block)
            except:
                log.warning("Block '{}' was not found".format(block_key))

        # This part no longer works now that we are using a Scene with GraphicsItems, but I'm sure there's still some way to do it
        # bounds = self.scene.itemsBoundingRect()
        # self.setSceneRect(bounds)
        # self.fitInView(bounds)

    def initEmpty(self):
        self.setSceneRect(0, 0, DEFAULT_MAX_X, DEFAULT_MAX_Y)

    def zoom(self, factor: float, anchor=QtWidgets.QGraphicsView.AnchorViewCenter):
        new_scalefactor = self.scalefactor * factor

        if new_scalefactor > 0.25 and new_scalefactor < 2.5:
            self.scalefactor = new_scalefactor
            self.setTransformationAnchor(anchor)
            self.setResizeAnchor(anchor)
            self.scale(factor, factor)

    def zoomOriginal(self):
        # TODO: Original scale factor as a constant?
        self.zoom(0.8 / self.scalefactor)

    def wheelEvent(self, event):
        # TODO: Support multi touch drag and drop for scrolling through the view
        if event.modifiers() == Qt.ControlModifier:
            factor = 1.1 if event.angleDelta().y() > 0 else (1.0 / 1.1)
            self.zoom(factor, anchor=QtWidgets.QGraphicsView.AnchorUnderMouse)

            # if new_scalefactor > 0.25 and new_scalefactor < 2.5:
            #     self.scalefactor = new_scalefactor
            #     self.setTransformationAnchor(QtWidgets.QGraphicsView.NoAnchor)
            #     self.setResizeAnchor(QtWidgets.QGraphicsView.NoAnchor)

            #     old_pos = self.mapToScene(event.pos())

            #     self.scale(factor, factor)
            #     new_pos = self.mapToScene(event.pos())

            #     delta = new_pos - old_pos
            #     self.translate(delta.x(), delta.y())

        elif event.modifiers() == Qt.ShiftModifier:
            self.horizontalScrollBar().setValue(
                self.horizontalScrollBar().value() - event.angleDelta().y())
        else:
            QtWidgets.QGraphicsView.wheelEvent(self, event)

    def mousePressEvent(self, event):
        if event.button() == Qt.LeftButton:
            self.mousePressed = True
            # This will pass the mouse move event to the scene
            super(FlowgraphView, self).mousePressEvent(event)

        elif event.button() == Qt.MiddleButton:
            self.isPanning = True
            self.dragPos = event.pos()
            self.setCursor(Qt.ClosedHandCursor)
            event.accept()

    def mouseMoveEvent(self, event):
        if self.isPanning:
            new_pos = event.pos()
            diff = new_pos - self.dragPos
            self.dragPos = new_pos
            self.horizontalScrollBar().setValue(
                self.horizontalScrollBar().value() - diff.x()
            )
            self.verticalScrollBar().setValue(
                self.verticalScrollBar().value() - diff.y()
            )
            event.accept()
        else:
            # This will pass the mouse move event to the scene
            super(FlowgraphView, self).mouseMoveEvent(event)

    def mouseReleaseEvent(self, event):
        if event.button() == Qt.LeftButton:
            self.mousePressed = False

        elif event.button() == Qt.MiddleButton:
            self.isPanning = False
            self.setCursor(Qt.ArrowCursor)
            event.accept()

        super(FlowgraphView, self).mouseReleaseEvent(event)

    def keyPressEvent(self, event):
        super(FlowgraphView, self).keyPressEvent(event)

    def keyReleaseEvent(self, event):
        super(FlowgraphView, self).keyPressEvent(event)

    def mouseDoubleClickEvent(self, event):
        # This will pass the double click event to the scene
        super(FlowgraphView, self).mouseDoubleClickEvent(event)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest
from gnuradio import blocks, zeromq
import pmt
import time


def make_tag(key, value, offset, srcid=None):
    tag = gr.tag_t()
    tag.key = pmt.string_to_symbol(key)
    tag.value = pmt.to_pmt(value)
    tag.offset = offset
    if srcid is not None:
        tag.srcid = pmt.to_pmt(srcid)
    return tag


def compare_tags(a, b):
    return a.offset == b.offset and pmt.equal(a.key, b.key) and \
        pmt.equal(a.value, b.value) and pmt.equal(a.srcid, b.srcid)


class qa_zeromq_pubsub (gr_unittest.TestCase):

    def setUp(self):
        self.send_tb = gr.top_block()
        self.recv_tb = gr.top_block()

    def tearDown(self):
        self.send_tb = None
        self.recv_tb = None

    def test_001(self):
        vlen = 10
        src_data = list(range(vlen)) * 100
        src = blocks.vector_source_f(src_data, False, vlen)
        zeromq_pub_sink = zeromq.pub_sink(
            gr.sizeof_float, vlen, "tcp://127.0.0.1:0", 0)
        address = zeromq_pub_sink.last_endpoint()
        zeromq_sub_source = zeromq.sub_source(
            gr.sizeof_float, vlen, address, 0)
        sink = blocks.vector_sink_f(vlen)
        self.send_tb.connect(src, zeromq_pub_sink)
        self.recv_tb.connect(zeromq_sub_source, sink)
        self.recv_tb.start()
        time.sleep(1.0)
        self.send_tb.start()
        time.sleep(1.0)
        self.recv_tb.stop()
        self.send_tb.stop()
        self.recv_tb.wait()
        self.send_tb.wait()
        self.assertFloatTuplesAlmostEqual(sink.data(), src_data)

    def test_002(self):
        # same as test_001, but insert a tag and set key filter
        vlen = 10
        src_data = list(range(vlen)) * 100

        src_tags = tuple([make_tag('key', 'val', 0, 'src'),
                          make_tag('key', 'val', 1, 'src')])

        src = blocks.vector_source_f(src_data, False, vlen, tags=src_tags)
        zeromq_pub_sink = zeromq.pub_sink(
            gr.sizeof_float,
            vlen,
            "tcp://127.0.0.1:0",
            0,
            pass_tags=True,
            key="filter_key")
        address = zeromq_pub_sink.last_endpoint()
        zeromq_sub_source = zeromq.sub_source(
            gr.sizeof_float, vlen, address, 0, pass_tags=True, key="filter_key")
        sink = blocks.vector_sink_f(vlen)
        self.send_tb.connect(src, zeromq_pub_sink)
        self.recv_tb.connect(zeromq_sub_source, sink)

        # start both flowgraphs
        self.recv_tb.start()
        time.sleep(1.0)
        self.send_tb.start()
        time.sleep(1.0)
        self.recv_tb.stop()
        self.send_tb.stop()
        self.recv_tb.wait()
        self.send_tb.wait()

        # compare data
        self.assertFloatTuplesAlmostEqual(sink.data(), src_data)

        # compare all tags
        rx_tags = sink.tags()
        self.assertEqual(len(src_tags), len(rx_tags))

        for in_tag, out_tag in zip(src_tags, rx_tags):
            self.assertTrue(compare_tags(in_tag, out_tag))


if __name__ == '__main__':
    gr_unittest.run(qa_zeromq_pubsub)



#
# Copyright 2010 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#
"""
Classes providing more user-friendly interfaces to the doxygen xml
docs than the generated classes provide.
"""

import os

from .generated import index
from .base import Base
from .text import description


class DoxyIndex(Base):
    """
    Parses a doxygen xml directory.
    """

    __module__ = "gnuradio.utils.doxyxml"

    def _parse(self):
        if self._parsed:
            return
        super(DoxyIndex, self)._parse()
        self._root = index.parse(os.path.join(self._xml_path, 'index.xml'))
        for mem in self._root.compound:
            converted = self.convert_mem(mem)
            # For files and namespaces we want the contents to be
            # accessible directly from the parent rather than having
            # to go through the file object.
            if self.get_cls(mem) == DoxyFile:
                if mem.name.endswith('.h'):
                    self._members += converted.members()
                    self._members.append(converted)
            elif self.get_cls(mem) == DoxyNamespace:
                self._members += converted.members()
                self._members.append(converted)
            else:
                self._members.append(converted)


class DoxyCompMem(Base):

    kind = None

    def __init__(self, *args, **kwargs):
        super(DoxyCompMem, self).__init__(*args, **kwargs)

    @classmethod
    def can_parse(cls, obj):
        return obj.kind == cls.kind

    def set_descriptions(self, parse_data):
        bd = description(getattr(parse_data, 'briefdescription', None))
        dd = description(getattr(parse_data, 'detaileddescription', None))
        self._data['brief_description'] = bd
        self._data['detailed_description'] = dd

    def set_parameters(self, data):
        vs = [ddc.value for ddc in data.detaileddescription.content_]
        pls = []
        for v in vs:
            if hasattr(v, 'parameterlist'):
                pls += v.parameterlist
        pis = []
        for pl in pls:
            pis += pl.parameteritem
        dpis = []
        for pi in pis:
            dpi = DoxyParameterItem(pi)
            dpi._parse()
            dpis.append(dpi)
        self._data['params'] = dpis


class DoxyCompound(DoxyCompMem):
    pass


class DoxyMember(DoxyCompMem):
    pass


class DoxyFunction(DoxyMember):

    __module__ = "gnuradio.utils.doxyxml"

    kind = 'function'

    def _parse(self):
        if self._parsed:
            return
        super(DoxyFunction, self)._parse()
        self.set_descriptions(self._parse_data)
        self.set_parameters(self._parse_data)
        if not self._data['params']:
            # If the params weren't set by a comment then just grab the names.
            self._data['params'] = []
            prms = self._parse_data.param
            for prm in prms:
                self._data['params'].append(DoxyParam(prm))

    brief_description = property(lambda self: self.data()['brief_description'])
    detailed_description = property(
        lambda self: self.data()['detailed_description'])
    params = property(lambda self: self.data()['params'])


Base.mem_classes.append(DoxyFunction)


class DoxyParam(DoxyMember):

    __module__ = "gnuradio.utils.doxyxml"

    def _parse(self):
        if self._parsed:
            return
        super(DoxyParam, self)._parse()
        self.set_descriptions(self._parse_data)
        self._data['declname'] = self._parse_data.declname

    @property
    def description(self):
        descriptions = []
        if self.brief_description:
            descriptions.append(self.brief_description)
        if self.detailed_description:
            descriptions.append(self.detailed_description)
        return '\n\n'.join(descriptions)

    brief_description = property(lambda self: self.data()['brief_description'])
    detailed_description = property(
        lambda self: self.data()['detailed_description'])
    name = property(lambda self: self.data()['declname'])


class DoxyParameterItem(DoxyMember):
    """A different representation of a parameter in Doxygen."""

    def _parse(self):
        if self._parsed:
            return
        super(DoxyParameterItem, self)._parse()
        names = []
        for nl in self._parse_data.parameternamelist:
            for pn in nl.parametername:
                names.append(description(pn))
        # Just take first name
        self._data['name'] = names[0]
        # Get description
        pd = description(self._parse_data.get_parameterdescription())
        self._data['description'] = pd

    description = property(lambda self: self.data()['description'])
    name = property(lambda self: self.data()['name'])


class DoxyClass(DoxyCompound):

    __module__ = "gnuradio.utils.doxyxml"

    kind = 'class'

    def _parse(self):
        if self._parsed:
            return
        super(DoxyClass, self)._parse()
        self.retrieve_data()
        if self._error:
            return
        self.set_descriptions(self._retrieved_data.compounddef)
        self.set_parameters(self._retrieved_data.compounddef)
        # Sectiondef.kind tells about whether private or public.
        # We just ignore this for now.
        self.process_memberdefs()

    brief_description = property(lambda self: self.data()['brief_description'])
    detailed_description = property(
        lambda self: self.data()['detailed_description'])
    params = property(lambda self: self.data()['params'])


Base.mem_classes.append(DoxyClass)


class DoxyFile(DoxyCompound):

    __module__ = "gnuradio.utils.doxyxml"

    kind = 'file'

    def _parse(self):
        if self._parsed:
            return
        super(DoxyFile, self)._parse()
        self.retrieve_data()
        self.set_descriptions(self._retrieved_data.compounddef)
        if self._error:
            return
        self.process_memberdefs()

    brief_description = property(lambda self: self.data()['brief_description'])
    detailed_description = property(
        lambda self: self.data()['detailed_description'])


Base.mem_classes.append(DoxyFile)


class DoxyNamespace(DoxyCompound):

    __module__ = "gnuradio.utils.doxyxml"

    kind = 'namespace'

    def _parse(self):
        if self._parsed:
            return
        super(DoxyNamespace, self)._parse()
        self.retrieve_data()
        self.set_descriptions(self._retrieved_data.compounddef)
        if self._error:
            return
        self.process_memberdefs()


Base.mem_classes.append(DoxyNamespace)


class DoxyGroup(DoxyCompound):

    __module__ = "gnuradio.utils.doxyxml"

    kind = 'group'

    def _parse(self):
        if self._parsed:
            return
        super(DoxyGroup, self)._parse()
        self.retrieve_data()
        if self._error:
            return
        cdef = self._retrieved_data.compounddef
        self._data['title'] = description(cdef.title)
        # Process inner groups
        grps = cdef.innergroup
        for grp in grps:
            converted = DoxyGroup.from_refid(grp.refid, top=self.top)
            self._members.append(converted)
        # Process inner classes
        klasses = cdef.innerclass
        for kls in klasses:
            converted = DoxyClass.from_refid(kls.refid, top=self.top)
            self._members.append(converted)
        # Process normal members
        self.process_memberdefs()

    title = property(lambda self: self.data()['title'])


Base.mem_classes.append(DoxyGroup)


class DoxyFriend(DoxyMember):

    __module__ = "gnuradio.utils.doxyxml"

    kind = 'friend'


Base.mem_classes.append(DoxyFriend)


class DoxyOther(Base):

    __module__ = "gnuradio.utils.doxyxml"

    kinds = set(['variable', 'struct', 'union', 'define', 'typedef', 'enum',
                 'dir', 'page', 'signal', 'slot', 'property'])

    @classmethod
    def can_parse(cls, obj):
        return obj.kind in cls.kinds


Base.mem_classes.append(DoxyOther)



#!/usr/bin/env python

MAX_NUM_FIELDS = 20

HEADER = """\
id: variable_struct
label: Struct Variable
flags: [ show_id ]

parameters:
"""

FUNCTIONCALL = """\
field{0}:value{0},\
"""

TEMPLATES = """\

templates:
    imports: "def struct(data): return type('Struct', (object,), data)()"
    var_make: |-
        self.${{id}} = ${{id}} = struct({{
            % for i in range({0}):
            <%
                field = context.get('field' + str(i))
                value = context.get('value' + str(i))
            %>
            % if len(str(field)) > 2:
            ${{field}}: ${{value}},
            % endif
            % endfor
        }})
"""

FIELD0 = """\
-   id: field0
    label: Field 0
    category: Fields
    dtype: string
    default: field0
    hide: part
"""

FIELDS = """\
-   id: field{0}
    label: Field {0}
    category: Fields
    dtype: string
    hide: part
"""

VALUES = """\
-   id: value{0}
    label: ${{field{0}}}
    dtype: raw
    default: '0'
    hide: ${{ 'none' if field{0} else 'all' }}
"""

ASSERTS = """\
- ${{ (str(field{0}) or "a")[0].isalpha() }}
- ${{ (str(field{0}) or "a").isalnum() }}
"""

FOOTER = """\

documentation: |-
    This is a simple struct/record like variable.

    Attribute/field names can be specified in the tab 'Fields'.
    For each non-empty field a parameter with type raw is shown.
    Value access via the dot operator, e.g. "variable_struct_0.field0"

file_format: 1
"""


def make_yml(num_fields):
    return ''.join((
        HEADER.format(num_fields),
        FIELD0, ''.join(FIELDS.format(i) for i in range(1, num_fields)),
        ''.join(VALUES.format(i) for i in range(num_fields)),'value: ${struct({',
        ''.join(FUNCTIONCALL.format(i) for i in range(num_fields)),'})}\n\nasserts:\n',
        ''.join(ASSERTS.format(i) for i in range(num_fields)),
        ''.join(TEMPLATES.format(num_fields)),
        FOOTER
    ))


if __name__ == '__main__':
    import sys
    try:
        filename = sys.argv[1]
    except IndexError:
        filename = __file__[:-3]

    data = make_yml(MAX_NUM_FIELDS)

    with open(filename, 'wb') as fp:
        fp.write(data.encode())



#!/usr/bin/env python
#
# Copyright 2021 NTESS LLC.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks, pdu
import pmt
import time


class qa_pdu_split (gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001_split(self):
        split = pdu.pdu_split()
        d1 = blocks.message_debug()
        d2 = blocks.message_debug()

        self.tb.msg_connect((split, 'dict'), (d1, 'store'))
        self.tb.msg_connect((split, 'vec'), (d2, 'store'))

        in_meta1 = pmt.dict_add(
            pmt.make_dict(), pmt.intern('num'), pmt.from_long(4))
        in_meta2 = pmt.dict_add(
            pmt.make_dict(), pmt.intern('n'), pmt.from_long(99))
        in_pdu = pmt.cons(in_meta1, pmt.init_u8vector(6, range(6)))

        self.tb.start()
        split.to_basic_block()._post(pmt.intern("pdu"), pmt.intern("MALFORMED PDU"))
        split.to_basic_block()._post(pmt.intern("pdu"), pmt.cons(
            pmt.PMT_NIL, pmt.init_u8vector(2, range(2))))
        split.to_basic_block()._post(pmt.intern(
            "pdu"), pmt.cons(in_meta2, pmt.init_u8vector(0, [])))
        split.to_basic_block()._post(pmt.intern("pdu"), in_pdu)
        split.to_basic_block()._post(pmt.intern("system"),
                                     pmt.cons(pmt.intern("done"), pmt.from_long(1)))
        self.waitFor(lambda: d1.num_messages() == 2,
                     timeout=1.0, poll_interval=0.01)
        self.waitFor(lambda: d2.num_messages() == 2,
                     timeout=1.0, poll_interval=0.01)
        self.tb.stop()
        self.tb.wait()

        self.assertTrue(pmt.equal(d1.get_message(0), in_meta2))
        self.assertTrue(pmt.equal(d1.get_message(1), in_meta1))
        self.assertTrue(pmt.equal(d2.get_message(
            0), pmt.init_u8vector(2, range(2))))
        self.assertTrue(pmt.equal(d2.get_message(
            1), pmt.init_u8vector(6, range(6))))

    def test_002_pass_empty(self):
        split = pdu.pdu_split(True)
        d1 = blocks.message_debug()
        d2 = blocks.message_debug()

        self.tb.msg_connect((split, 'dict'), (d1, 'store'))
        self.tb.msg_connect((split, 'vec'), (d2, 'store'))

        in_meta1 = pmt.dict_add(
            pmt.make_dict(), pmt.intern('num'), pmt.from_long(4))
        in_meta2 = pmt.dict_add(
            pmt.make_dict(), pmt.intern('n'), pmt.from_long(99))
        in_pdu = pmt.cons(in_meta1, pmt.init_u8vector(6, range(6)))

        self.tb.start()
        split.to_basic_block()._post(pmt.intern("pdu"), pmt.intern("MALFORMED PDU"))
        split.to_basic_block()._post(pmt.intern("pdu"), pmt.cons(
            pmt.PMT_NIL, pmt.init_u8vector(2, range(2))))
        split.to_basic_block()._post(pmt.intern(
            "pdu"), pmt.cons(in_meta2, pmt.init_u8vector(0, [])))
        split.to_basic_block()._post(pmt.intern("pdu"), in_pdu)
        split.to_basic_block()._post(pmt.intern("system"),
                                     pmt.cons(pmt.intern("done"), pmt.from_long(1)))
        self.waitFor(lambda: d1.num_messages() == 3,
                     timeout=1.0, poll_interval=0.01)
        self.waitFor(lambda: d2.num_messages() == 3,
                     timeout=1.0, poll_interval=0.01)
        self.tb.wait()

        self.assertTrue(pmt.equal(d1.get_message(0), pmt.PMT_NIL))
        self.assertTrue(pmt.equal(d1.get_message(1), in_meta2))
        self.assertTrue(pmt.equal(d1.get_message(2), in_meta1))
        self.assertTrue(pmt.equal(d2.get_message(
            0), pmt.init_u8vector(2, range(2))))
        self.assertTrue(pmt.equal(d2.get_message(1), pmt.init_u8vector(0, [])))
        self.assertTrue(pmt.equal(d2.get_message(
            2), pmt.init_u8vector(6, range(6))))


if __name__ == '__main__':
    gr_unittest.run(qa_pdu_split)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2014 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest
from gnuradio import blocks, zeromq
from gnuradio import eng_notation
import time


class qa_zeromq_reqrep (gr_unittest.TestCase):

    def setUp(self):
        self.send_tb = gr.top_block()
        self.recv_tb = gr.top_block()

    def tearDown(self):
        self.send_tb = None
        self.recv_tb = None

    def test_001(self):
        vlen = 10
        src_data = list(range(vlen)) * 100
        src = blocks.vector_source_f(src_data, False, vlen)
        zeromq_rep_sink = zeromq.rep_sink(
            gr.sizeof_float, vlen, "tcp://127.0.0.1:0", 0)
        address = zeromq_rep_sink.last_endpoint()
        zeromq_req_source = zeromq.req_source(
            gr.sizeof_float, vlen, address, 0)
        sink = blocks.vector_sink_f(vlen)
        self.send_tb.connect(src, zeromq_rep_sink)
        self.recv_tb.connect(zeromq_req_source, sink)
        self.recv_tb.start()
        time.sleep(1.0)
        self.send_tb.start()
        time.sleep(1.0)
        self.recv_tb.stop()
        self.send_tb.stop()
        self.recv_tb.wait()
        self.send_tb.wait()
        self.assertFloatTuplesAlmostEqual(sink.data(), src_data)


if __name__ == '__main__':
    gr_unittest.run(qa_zeromq_reqrep)



#
# Copyright 2010 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

"""
Miscellaneous utilities for managing mods and demods, as well as other items
useful in dealing with generalized handling of different modulations and demods.
"""

import inspect


# Type 1 modulators accept a stream of bytes on their input and produce complex baseband output
_type_1_modulators = {}


def type_1_mods():
    return _type_1_modulators


def add_type_1_mod(name, mod_class):
    _type_1_modulators[name] = mod_class


# Type 1 demodulators accept complex baseband input and produce a stream of bits, packed
# 1 bit / byte as their output.  Their output is completely unambiguous.  There is no need
# to resolve phase or polarity ambiguities.
_type_1_demodulators = {}


def type_1_demods():
    return _type_1_demodulators


def add_type_1_demod(name, demod_class):
    _type_1_demodulators[name] = demod_class


# Also record the constellation making functions of the modulations
_type_1_constellations = {}


def type_1_constellations():
    return _type_1_constellations


def add_type_1_constellation(name, constellation):
    _type_1_constellations[name] = constellation


def extract_kwargs_from_options(function, excluded_args, options):
    """
    Given a function, a list of excluded arguments and the result of
    parsing command line options, create a dictionary of key word
    arguments suitable for passing to the function.  The dictionary
    will be populated with key/value pairs where the keys are those
    that are common to the function's argument list (minus the
    excluded_args) and the attributes in options.  The values are the
    corresponding values from options unless that value is None.
    In that case, the corresponding dictionary entry is not populated.

    (This allows different modulations that have the same parameter
    names, but different default values to coexist.  The downside is
    that --help in the option parser will list the default as None,
    but in that case the default provided in the __init__ argument
    list will be used since there is no kwargs entry.)

    Args:
        function: the function whose parameter list will be examined
        excluded_args: function arguments that are NOT to be added to the dictionary (sequence of strings)
        options: result of command argument parsing (optparse.Values)
    """

    # Try this in C++ ;)
    spec = inspect.getfullargspec(function)
    d = {}
    for kw in [a for a in spec.args if a not in excluded_args]:
        if hasattr(options, kw):
            if getattr(options, kw) is not None:
                d[kw] = getattr(options, kw)
    return d


def extract_kwargs_from_options_for_class(cls, options):
    """
    Given command line options, create dictionary suitable for passing to __init__
    """
    d = extract_kwargs_from_options(
        cls.__init__, ('self',), options)
    for base in cls.__bases__:
        if hasattr(base, 'extract_kwargs_from_options'):
            d.update(base.extract_kwargs_from_options(options))
    return d



#!/usr/bin/env python
#
# Copyright 2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import math

from gnuradio import gr, gr_unittest, analog, blocks


def sincos(x):
    return math.cos(x) + math.sin(x) * 1j


class test_phase_modulator(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_fm_001(self):
        pi = math.pi
        sensitivity = pi / 4
        src_data = (1.0 / 4, 1.0 / 2, 1.0 / 4, -1.0 / 4, -1.0 / 2, -1 / 4.0)
        expected_result = tuple([sincos(sensitivity * x) for x in src_data])

        src = blocks.vector_source_f(src_data)
        op = analog.phase_modulator_fc(sensitivity)
        dst = blocks.vector_sink_c()

        self.tb.connect(src, op)
        self.tb.connect(op, dst)
        self.tb.run()

        result_data = dst.data()
        self.assertComplexTuplesAlmostEqual(expected_result, result_data, 5)


if __name__ == '__main__':
    gr_unittest.run(test_phase_modulator)



#!/usr/bin/env python
#
# Copyright 2021 NTESS LLC.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks, pdu
import pmt
import time


class qa_add_system_time(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

        self.add_sys_time = pdu.add_system_time(pmt.intern('systime'))
        self.debug = blocks.message_debug()

        # make connections
        self.tb.msg_connect((self.add_sys_time, 'pdu'), (self.debug, 'store'))
        self.tb.msg_connect((self.add_sys_time, 'pdu'), (self.debug, 'print'))

    def tearDown(self):
        self.tb = None

    def test_001_basic_io(self):
        self.tb.start()
        # provide two non PDU inputs and one PDU input
        self.add_sys_time.to_basic_block()._post(
            pmt.intern("pdu"), pmt.intern("BAD PDU"))
        self.add_sys_time.to_basic_block()._post(pmt.intern("pdu"),
                                                 pmt.cons(pmt.from_long(4), pmt.PMT_NIL))
        self.add_sys_time.to_basic_block()._post(pmt.intern("pdu"),
                                                 pmt.cons(pmt.make_dict(), pmt.init_f32vector(1, [0.0])))
        self.waitFor(lambda: self.debug.num_messages() >=
                     1, timeout=1.0, poll_interval=0.01)
        self.tb.stop()
        self.tb.wait()

        # make sure we got one message and it has a systime key
        self.assertEqual(1, self.debug.num_messages())
        self.assertTrue(pmt.dict_has_key(
            pmt.car(self.debug.get_message(0)), pmt.intern('systime')))

    def test_002_timing(self):
        self.tb.start()

        self.add_sys_time.to_basic_block()._post(
            pmt.intern("pdu"), pmt.intern("BAD PDU"))
        self.add_sys_time.to_basic_block()._post(pmt.intern("pdu"),
                                                 pmt.cons(pmt.make_dict(), pmt.init_u8vector(1, [0])))
        # wait for one second to provide a time difference between messages
        time.sleep(1.0)
        self.add_sys_time.to_basic_block()._post(pmt.intern("pdu"),
                                                 pmt.cons(pmt.make_dict(), pmt.init_u8vector(1, [0])))
        self.waitFor(lambda: self.debug.num_messages() ==
                     2, timeout=1.0, poll_interval=0.01)
        self.tb.stop()
        self.tb.wait()

        t0 = pmt.to_double(pmt.dict_ref(pmt.car(self.debug.get_message(0)),
                                        pmt.intern("systime"),
                                        pmt.from_double(0.0)))
        t1 = pmt.to_double(pmt.dict_ref(pmt.car(self.debug.get_message(1)),
                                        pmt.intern("systime"),
                                        pmt.from_double(0.0)))
        # should be sufficient tolerance
        self.assertAlmostEqual(t0, t1 - 1, delta=0.5)


if __name__ == '__main__':
    gr_unittest.run(qa_add_system_time)



#!/usr/bin/env python
#
# Copyright 2007,2010,2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, digital, blocks


class test_glfsr_source(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_000_make_b(self):
        src = digital.glfsr_source_b(16)
        self.assertEqual(src.mask(), 0x8016)
        self.assertEqual(src.period(), 2**16 - 1)

    def test_001_degree_b(self):
        self.assertRaises(RuntimeError,
                          lambda: digital.glfsr_source_b(0))
        self.assertRaises(RuntimeError,
                          lambda: digital.glfsr_source_b(65))

    def test_002_correlation_b(self):
        for degree in range(
                1, 11):                # Higher degrees take too long to correlate
            src = digital.glfsr_source_b(degree, False)
            b2f = digital.chunks_to_symbols_bf((-1.0, 1.0), 1)
            dst = blocks.vector_sink_f()
            del self.tb  # Discard existing top block
            self.tb = gr.top_block()
            self.tb.connect(src, b2f, dst)
            self.tb.run()
            self.tb.disconnect_all()
            actual_result = dst.data()
            R = auto_correlate(actual_result)
            # Auto-correlation peak at origin
            self.assertEqual(R[0], float(len(R)))
            for i in range(len(R) - 1):
                # Auto-correlation minimum everywhere else
                self.assertEqual(R[i + 1], -1.0)

    def test_003_make_f(self):
        src = digital.glfsr_source_f(16)
        self.assertEqual(src.mask(), 0x8016)
        self.assertEqual(src.period(), 2**16 - 1)

    def test_004_degree_f(self):
        self.assertRaises(RuntimeError,
                          lambda: digital.glfsr_source_f(0))
        self.assertRaises(RuntimeError,
                          lambda: digital.glfsr_source_f(65))

    def test_005_correlation_f(self):
        for degree in range(
                1, 11):                # Higher degrees take too long to correlate
            src = digital.glfsr_source_f(degree, False)
            dst = blocks.vector_sink_f()
            del self.tb  # Discard existing top block
            self.tb = gr.top_block()
            self.tb.connect(src, dst)
            self.tb.run()

            actual_result = dst.data()
            R = auto_correlate(actual_result)
            # Auto-correlation peak at origin
            self.assertEqual(R[0], float(len(R)))
            for i in range(len(R) - 1):
                # Auto-correlation minimum everywhere else
                self.assertEqual(R[i + 1], -1.0)


def auto_correlate(data):
    l = len(data)
    R = [0, ] * l
    for lag in range(l):
        for i in range(l):
            R[lag] += data[i] * data[i - lag]
    return R


if __name__ == '__main__':
    gr_unittest.run(test_glfsr_source)



#!/usr/bin/env python
#
# Copyright 2022,2023 Marcus Müller
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

from gnuradio import gr, gr_unittest
import numpy
import time


class logging_blk(gr.sync_block):
    def __init__(self, name=""):
        gr.sync_block.__init__(self, name, in_sig=[], out_sig=[numpy.float32])

        self.logger.info("Initialized")

    def start(self):
        self.logger.info("Started")

    def stop(self):
        self.logger.info("Stopped")

    def work(self, _, out_streams):
        if self.nitems_written(0) < 10**6:
            return len(out_streams[0])
        return -1


class drop_sink(gr.sync_block):
    def __init__(self, name=""):
        gr.sync_block.__init__(self, name, in_sig=[numpy.float32], out_sig=[])

    def work(self, in_streams, _):
        return len(in_streams[0])


class test_logging(gr_unittest.TestCase):
    def setUp(self):
        self.blockname = "logger test"
        self.backend = gr.dictionary_logger_backend()
        gr.logging().add_default_sink(self.backend)

        self.tb = gr.top_block()
        self.blk = logging_blk(self.blockname)
        self.sink = drop_sink()

        self.tb.connect(self.blk, self.sink)

    def tearDown(self):
        self.tb = None

    def test_000_level(self):
        levels = gr.log_levels

        for level in (levels.debug, levels.info, levels.warn, levels.err, levels.critical):
            gr.logging().set_default_level(level)
            self.assertEqual(gr.logging().default_level(), level, f"not working for {level}")
        gr.logging().set_default_level(levels.info)

    def test_001_log(self):
        gr.logging().set_default_level(gr.log_levels.debug)
        self.tb.run(max_noutput_items=1024)
        time.sleep(0.1)
        logged = self.backend.get_map()
        my_block_log = logged[self.blockname]
        expected = ("Initialized", "Started", "Stopped")
        for expectation, log_entry in zip(expected, sorted(my_block_log)):
            self.assertEqual(expectation, log_entry[1])


class test_disabled_logging(gr_unittest.TestCase):
    def setUp(self):
        self.blockname = "logger test"
        gr.logging().set_default_level(gr.log_levels.off)
        self.backend = gr.dictionary_logger_backend()
        gr.logging().add_default_sink(self.backend)

        self.tb = gr.top_block()
        self.blk = logging_blk(self.blockname)
        self.sink = drop_sink()

        self.tb.connect(self.blk, self.sink)

    def test_001_log_disabled(self):
        self.tb.run(max_noutput_items=1024)
        time.sleep(0.1)
        logged = self.backend.get_map()
        self.assertNotIn(self.blockname, logged)


class logging_hier(gr.hier_block2):
    MAGIC_STRING = "magic value"

    def __init__(self):
        gr.hier_block2.__init__(self, "python hier", gr.io_signature(0, 0, 0), gr.io_signature(0, 0, 0))
        self.logger.error(self.MAGIC_STRING)


class test_hier_logging(gr_unittest.TestCase):
    def setUp(self):
        self.blockname = "Py Hier Blk python hier"
        self.backend = gr.dictionary_logger_backend()
        gr.logging().add_default_sink(self.backend)

    def tearDown(self):
        pass

    def test_001_log(self):
        gr.logging().set_default_level(gr.log_levels.debug)
        self.dut = logging_hier()
        time.sleep(0.1)
        logged = self.backend.get_map()
        my_block_log = logged[self.blockname]
        expected = (self.dut.MAGIC_STRING,)
        for expectation, log_entry in zip(expected, sorted(my_block_log)):
            self.assertEqual(expectation, log_entry[1])


class logging_tb(gr.top_block):
    BLOCKNAME = "Aqualung"

    def __init__(self, init_msgs=None, start_msgs=None, stop_msgs=None):
        gr.top_block.__init__(self, self.BLOCKNAME)
        self.init_msgs = init_msgs or []
        self.start_msgs = start_msgs or []
        self.stop_msgs = stop_msgs or []

        for msg in self.init_msgs:
            self.logger.debug("init " + msg)

    def start(self, *args, **kwargs):
        for msg in self.start_msgs:
            self.logger.debug("start " + msg)
        gr.top_block.__init__(self, *args, **kwargs)

    def stop(self, *args, **kwargs):
        for msg in self.stop_msgs:
            self.logger.debug("stop " + msg)
        gr.top_block.__init__(self, *args, **kwargs)


class test_topblock_logging(gr_unittest.TestCase):
    def setUp(self):
        gr.logging().set_default_level(gr.log_levels.debug)
        self.backend = gr.dictionary_logger_backend()
        gr.logging().add_default_sink(self.backend)
        self.init_msgs = ["1", "2", "3"]
        self.start_msgs = ["A", "B", "C"]
        self.stop_msgs = ["D", "E", "F"]
        self.tb = logging_tb(init_msgs=self.init_msgs, start_msgs=self.start_msgs, stop_msgs=self.stop_msgs)
        self.blockname = f"Python Top Blk {self.tb.BLOCKNAME}"

    def test_001_log_init(self):
        logged = self.backend.get_map()
        print(logged)
        my_block_log = logged[self.blockname]
        expected = self.init_msgs
        for expectation, log_entry in zip(expected, sorted(my_block_log)):
            self.assertEqual("init " + expectation, log_entry[1])

    def test_002_log_start(self):
        self.tb.start()
        logged = self.backend.get_map()
        print(logged)
        my_block_log = [msg for msg in logged[self.blockname] if (not msg[1].startswith("init "))]
        expected = self.start_msgs
        for expectation, log_entry in zip(expected, sorted(my_block_log)):
            self.assertEqual("start " + expectation, log_entry[1])

    def test_003_log_stop(self):
        self.tb.stop()
        logged = self.backend.get_map()
        print(logged)
        my_block_log = [
            msg for msg in logged[self.blockname] if not (msg[1].startswith("init ") or msg[1].startswith("start "))
        ]
        expected = self.stop_msgs
        for expectation, log_entry in zip(expected, sorted(my_block_log)):
            self.assertEqual("stop " + expectation, log_entry[1])


if __name__ == "__main__":
    gr_unittest.run(test_logging)
